#!/usr/bin/python

import json
import logging
import os
import unittest
from copy import deepcopy
from unittest.mock import MagicMock

from mcrit.index.MinHashIndex import MinHashIndex
from mcrit.matchers.MatcherInterface import IS_LIBRARY_FLAG, IS_MINHASH_FLAG, IS_PICHASH_FLAG
from mcrit.matchers.MatcherQuery import MatcherQuery
from mcrit.matchers.MatcherSample import MatcherSample

# from mcrit.storage.MemoryStorage import MemoryStorage
from mcrit.matchers.MatcherVs import MatcherVs
from mcrit.storage.SampleEntry import SampleEntry
from smda.common.SmdaFunction import SmdaFunction
from smda.common.SmdaReport import SmdaReport

from .context import config

LOG = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO, format="%(asctime)-15s %(message)s")
logging.disable(logging.CRITICAL)


class MatcherTestSuite(unittest.TestCase):
    """Run a full example on a memory dump"""

    # pichash_expected = {'pichash_summary': {'num_own_functions_matched': 0, 'num_foreign_functions_matched': 0, 'num_own_functions_matched_as_library': 0, 'num_self_matches': 0, 'bytes_matched': 0}, 'pichash_matches': {}}

    def __init__(self, *args, **kwargs):
        super(MatcherTestSuite, self).__init__(*args, **kwargs)

        THIS_FILE_PATH = str(os.path.abspath(__file__))
        PROJECT_ROOT = str(os.path.abspath(os.sep.join([THIS_FILE_PATH, "..", ".."])))
        example_file_path_1 = os.sep.join([PROJECT_ROOT, "tests", "example_report.smda"])
        example_file_path_2 = os.sep.join([PROJECT_ROOT, "tests", "example_report_2.smda"])
        example_file_path_3 = os.sep.join([PROJECT_ROOT, "tests", "example_report_3.smda"])
        library_file_path = os.sep.join([PROJECT_ROOT, "tests", "library_report.smda"])
        self.smda_report_1 = SmdaReport.fromFile(example_file_path_1)
        self.smda_report_2 = SmdaReport.fromFile(example_file_path_2)
        self.smda_report_2.family = "test_family"
        self.smda_report_3 = SmdaReport.fromFile(example_file_path_3)
        self.smda_report_3.family = "test_family_b"
        self.library_report = SmdaReport.fromFile(library_file_path)

        function_1_selected = list(self.smda_report_1.getFunctions())[3]
        function_2_selected = list(self.smda_report_2.getFunctions())[5]
        offset = function_2_selected.offset
        # self.smda_report_2.xcfg[offset] = function_1_selected
        function_2_selected.pic_hash = function_1_selected.pic_hash

        # make a selfmatch
        self.smda_report_1.xcfg[list(self.smda_report_1.getFunctions())[1].offset] = list(
            self.smda_report_1.getFunctions()
        )[5]

    # from matchervs
    function_matches_expected = [
        {
            "fid": 9,
            "matches": [
                (0, 2, 19, 100.0, IS_MINHASH_FLAG + IS_PICHASH_FLAG + IS_LIBRARY_FLAG),
                (1, 0, 0, 84.375, IS_MINHASH_FLAG),
                (2, 3, 22, 84.375, IS_MINHASH_FLAG),
            ],
            "num_bytes": 354.0,
            "num_instructions": 120,
            "offset": 0,
        },
        {
                "num_bytes": 35.0,
                "num_instructions": 11,
                "offset": 2220,
                "matches": [],
                "fid": 10
        },
        {
            "fid": 11,
            "matches": [
                (0, 2, 20, 100.0, IS_MINHASH_FLAG + IS_PICHASH_FLAG + IS_LIBRARY_FLAG),
                (1, 0, 1, 92.1875, IS_MINHASH_FLAG),
            ],
            "num_bytes": 638.0,
            "num_instructions": 207,
            "offset": 364,
        },
        {
            "fid": 12,
            "matches": [
                (0, 2, 21, 100.0, IS_MINHASH_FLAG + IS_PICHASH_FLAG + IS_LIBRARY_FLAG),
                (1, 0, 5, 100.0, IS_MINHASH_FLAG + IS_PICHASH_FLAG)
            ],
            "num_bytes": 166.0,
            "num_instructions": 64,
            "offset": 1004,
        },
        {
            "fid": 13,
            "matches": [(1, 0, 3, 67.1875, IS_MINHASH_FLAG)],
            "num_bytes": 1047.0,
            "num_instructions": 365,
            "offset": 1172,
        },
        {
            "num_bytes": 35.0,
            "num_instructions": 11,
            "offset": 2220,
            "matches": [],
            "fid": 14
        },
        {
            "num_bytes": 524.0,
            "num_instructions": 159,
            "offset": 2256,
            "matches": [],
            "fid": 15
        },
        {
            "fid": 16,
            "matches": [(1, 0, 5, 84.375, IS_MINHASH_FLAG)],
            "num_bytes": 915.0,
            "num_instructions": 287,
            "offset": 2780,
        },
        {
            "fid": 17,
            "matches": [(1, 0, 6, 98.4375, IS_MINHASH_FLAG)],
            "num_bytes": 727.0,
            "num_instructions": 226,
            "offset": 3696,
        },
        {
            "fid": 18,
            "matches": [(1, 0, 7, 67.1875, IS_MINHASH_FLAG)],
            "num_bytes": 1850.0,
            "num_instructions": 543,
            "offset": 4424,
        },
    ]

    function_matches_expected_vs = [
        {
            "fid": 9,
            "matches": [
                (1, 0, 0, 84.375, IS_MINHASH_FLAG),
            ],
            "num_bytes": 354.0,
            "num_instructions": 120,
            "offset": 0,
        },
        {
            "num_bytes": 35.0,
            "num_instructions": 11,
            "offset": 2220,
            "matches": [],
            "fid": 10
        },
        {
            "fid": 11,
            "matches": [
                (1, 0, 1, 92.1875, IS_MINHASH_FLAG),
            ],
            "num_bytes": 638.0,
            "num_instructions": 207,
            "offset": 364,
        },
        {
            "fid": 12,
            "matches": [
                (1, 0, 5, 100.0, IS_MINHASH_FLAG + IS_PICHASH_FLAG),
            ],
            "num_bytes": 166.0,
            "num_instructions": 64,
            "offset": 1004,
        },
        {
            "fid": 13,
            "matches": [(1, 0, 3, 67.1875, IS_MINHASH_FLAG)],
            "num_bytes": 1047.0,
            "num_instructions": 365,
            "offset": 1172,
        },
        {
            "num_bytes": 35.0,
            "num_instructions": 11,
            "offset": 2220,
            "matches": [],
            "fid": 14
        },
        {
            "num_bytes": 524.0,
            "num_instructions": 159,
            "offset": 2256,
            "matches": [],
            "fid": 15
        },
        {
            "fid": 16,
            "matches": [(1, 0, 5, 84.375, IS_MINHASH_FLAG)],
            "num_bytes": 915.0,
            "num_instructions": 287,
            "offset": 2780,
        },
        {
            "fid": 17,
            "matches": [(1, 0, 6, 98.4375, IS_MINHASH_FLAG)],
            "num_bytes": 727.0,
            "num_instructions": 226,
            "offset": 3696,
        },
        {
            "fid": 18,
            "matches": [(1, 0, 7, 67.1875, IS_MINHASH_FLAG)],
            "num_bytes": 1850.0,
            "num_instructions": 543,
            "offset": 4424,
        },
    ]

    minhash_aggregation_expected = {
        "num_own_functions_matched": 7,
        "num_foreign_functions_matched": 10,
        "num_own_functions_matched_as_library": 3,
        "num_self_matches": 2,
        "bytes_matched": 5697.0,
    }

    minhash_aggregation_expected_vs = {
        "num_own_functions_matched": 7,
        "num_foreign_functions_matched": 6,
        "num_own_functions_matched_as_library": 0,
        "num_self_matches": 2,
        "bytes_matched": 5697.0,
    }

    minhash_aggregation_expected_query = deepcopy(minhash_aggregation_expected)
    minhash_aggregation_expected_query["num_self_matches"] = 0

    pichash_aggregation_expected = {
        "num_own_functions_matched": 3,
        "num_foreign_functions_matched": 4,
        "num_own_functions_matched_as_library": 3,
        "num_self_matches": 2,
        "bytes_matched": 1158.0,
    }

    pichash_aggregation_expected_query = deepcopy(pichash_aggregation_expected)
    pichash_aggregation_expected_query["num_self_matches"] = 0

    pichash_aggregation_expected_vs = {
        "num_own_functions_matched": 1,
        "num_foreign_functions_matched": 1,
        "num_own_functions_matched_as_library": 0,
        "num_self_matches": 2,
        "bytes_matched": 166.0,
    }
    maxDiff = None
    # sample2
    # TODO mark s.th as library,
    # TODO make s.th a pic+min match
    sample_summary_entry_2_expected = {
        "sample_id": 0,
        "family": "test_family",
        "family_id": 1,
        "version": "",
        "bitness": 32,
        "sha256": "39401c7f9518f9710bfd6d64e13a5bf4efdec42e54c9de363b4871e8f374b579",
        "filename": "",
        "num_bytes": 6158.0,
        "num_functions": 9,
        'is_library': False,
        "matched": {
            "functions": {
                "combined": 7,
                "library": 3,
                "minhashes": 7,
                "pichashes": 1,
            },
            "bytes": {
                "unweighted": 5697.0,
                "score_weighted": 4486.9375,
                "frequency_weighted": 4337.59375,
                "nonlib_unweighted": 4539.0,
                "nonlib_score_weighted": 3434.09375,
                "nonlib_frequency_weighted": 3434.09375
            },
            "percent": {
                "unweighted": 90.55793991416309,
                "score_weighted": 71.32312033063106,
                "frequency_weighted": 68.94919329200445,
                "nonlib_unweighted": 88.42781998831093,
                "nonlib_score_weighted": 66.90227449834404,
                "nonlib_frequency_weighted": 66.90227449834404,
            },
        },
    }

    sample_summary_entry_3_expected = {
        "sample_id": 3,
        "family": "test_family_b",
        "family_id": 2,
        "version": "",
        "bitness": 32,
        "sha256": "39401c7f9518f9710bfd6d64e13a5bf4efdec42e54c9de363b4871e8f374b57f",
        "filename": "",
        "num_bytes": 351.0,
        "num_functions": 9,
        'is_library': False,
        "matched": {
            "functions": {
                "combined": 1,
                "library": 1,
                "minhashes": 1,
                "pichashes": 0,
            },
            "bytes": {
                "unweighted": 354.0,
                "score_weighted": 298.6875,
                "frequency_weighted": 149.34375,
                "nonlib_unweighted": 0,
                "nonlib_score_weighted": 0,
                "nonlib_frequency_weighted": 0
            },
            "percent": {
                "unweighted": 5.627086313781593,
                "score_weighted": 4.747854077253219,
                "frequency_weighted": 2.3739270386266096,
                "nonlib_unweighted": 0.0,
                "nonlib_score_weighted": 0.0,
                "nonlib_frequency_weighted": 0.0
            },
        },
    }

    sample_summary_entry_expected_vs = {
        "sample_id": 0,
        "family": "test_family",
        "family_id": 1,
        "version": "",
        "bitness": 32,
        "sha256": "39401c7f9518f9710bfd6d64e13a5bf4efdec42e54c9de363b4871e8f374b579",
        "filename": "",
        "num_bytes": 6158.0,
        "num_functions": 9,
        'is_library': False,
        "matched": {
            "functions": {
                "combined": 7,
                "library": 0,
                "minhashes": 7,
                "pichashes": 1,
            },
            "bytes": {
                "unweighted": 5697.0,
                "score_weighted": 4486.9375,
                "frequency_weighted": 4486.9375,
                "nonlib_unweighted": 5697.0,
                "nonlib_score_weighted": 4486.9375,
                "nonlib_frequency_weighted": 4486.9375
            },
            "percent": {
                "unweighted": 90.55793991416309,
                "score_weighted": 71.32312033063106,
                "frequency_weighted": 71.32312033063106,
                "nonlib_unweighted": 90.55793991416309,
                "nonlib_score_weighted": 71.32312033063106,
                "nonlib_frequency_weighted": 71.32312033063106
            }
        },
    }

    sample_summary_lib_entry_expected = {
        "sample_id": 2,
        "family": "",
        "family_id": 0,
        "version": "",
        "bitness": 32,
        "sha256": "ae38ff0778fb8dfa1deb17301a15165934312648d232d167cd0c0034c24689e2",
        "filename": "",
        "num_bytes": 1158.0,
        "num_functions": 3,
        'is_library': True,
        "matched": {
            "functions": {
                "combined": 3,
                "library": 3,
                "minhashes": 3,
                "pichashes": 3,
            },
            "bytes": {
                "unweighted": 1158.0,
                "score_weighted": 1158.0,
                "frequency_weighted": 981.0,
                "nonlib_unweighted": 0,
                "nonlib_score_weighted": 0,
                "nonlib_frequency_weighted": 0,
            },
            "percent": {
                "unweighted": 18.407248450166904,
                "score_weighted": 18.407248450166904,
                "frequency_weighted": 15.593705293276109,
                "nonlib_unweighted": 0.0,
                "nonlib_score_weighted": 0.0,
                "nonlib_frequency_weighted": 0.0
            },
        },
    }

    def testMatcherVs(self):
        index = MinHashIndex(config=config)
        worker = index.queue._worker

        sampleEntry2 = index._storage.addSmdaReport(self.smda_report_2)
        id2 = sampleEntry2.sample_id

        sampleEntry1 = index._storage.addSmdaReport(self.smda_report_1)
        id1 = sampleEntry1.sample_id

        libraryEntry = index._storage.addSmdaReport(self.library_report)
        id_lib = libraryEntry.sample_id

        sampleEntry3 = index._storage.addSmdaReport(self.smda_report_3)
        id3 = sampleEntry3.sample_id

        worker.updateMinHashesForSample(id1)
        worker.updateMinHashesForSample(id2)
        worker.updateMinHashesForSample(id_lib)
        worker.updateMinHashesForSample(id3)

        matcher = MatcherVs(worker)
        result = matcher.getMatchesForSample(id1, id2)

        self.assertEqual(result["info"]["sample"], sampleEntry1.toDict())
        self.assertEqual(result["other_sample_info"], sampleEntry2.toDict())
        self.assertTrue("duration" in result["info"]["job"])
        self.assertTrue("timestamp" in result["info"]["job"])
        self.assertTrue(isinstance(result["info"]["job"]["timestamp"], str))
        self.assertTrue(len(result["info"]["job"]["timestamp"]) > 0)
        self.assertNotEqual(result["info"]["job"]["duration"], 0)

        self.assertEqual(result["matches"]["aggregation"]["pichash"], self.pichash_aggregation_expected_vs)
        self.assertEqual(result["matches"]["aggregation"]["minhash"], self.minhash_aggregation_expected_vs)
        self.assertEqual(result["matches"]["functions"], self.function_matches_expected_vs)
        self.assertEqual(result["matches"]["samples"], [self.sample_summary_entry_expected_vs])

    def testMatcherSample(self):
        index = MinHashIndex(config=config)
        worker = index.queue._worker

        sampleEntry2 = index._storage.addSmdaReport(self.smda_report_2)
        id2 = sampleEntry2.sample_id

        sampleEntry1 = index._storage.addSmdaReport(self.smda_report_1)
        id1 = sampleEntry1.sample_id

        libraryEntry = index._storage.addSmdaReport(self.library_report)
        id_lib = libraryEntry.sample_id

        sampleEntry3 = index._storage.addSmdaReport(self.smda_report_3)
        id3 = sampleEntry3.sample_id

        # a=set([fe.pichash for fe in function_entries_1])
        # b=set([fe.pichash for fe in function_entries_2])
        # print(set.intersection(a,b))

        worker.updateMinHashesForSample(id1)
        worker.updateMinHashesForSample(id2)
        worker.updateMinHashesForSample(id3)
        worker.updateMinHashesForSample(id_lib)

        matcher = MatcherSample(worker)
        result = matcher.getMatchesForSample(id1)

        print(json.dumps(result, indent=4))

        self.assertEqual(result["info"]["sample"], sampleEntry1.toDict())
        self.assertTrue("duration" in result["info"]["job"])
        self.assertNotEqual(result["info"]["job"]["duration"], 0)
        self.assertTrue("timestamp" in result["info"]["job"])
        self.assertTrue(isinstance(result["info"]["job"]["timestamp"], str))
        self.assertTrue(len(result["info"]["job"]["timestamp"]) > 0)

        self.assertEqual(result["matches"]["aggregation"]["minhash"], self.minhash_aggregation_expected)
        self.assertEqual(result["matches"]["aggregation"]["pichash"], self.pichash_aggregation_expected)
        self.assertEqual(result["matches"]["functions"], self.function_matches_expected)
        self.maxDiff = None
        self.assertEqual(
            result["matches"]["samples"],
            [
                self.sample_summary_lib_entry_expected,
                self.sample_summary_entry_2_expected,
                self.sample_summary_entry_3_expected,
            ],
        )

    def testMatcherQuery(self):
        index = MinHashIndex(config=config)
        worker = index.queue._worker

        # sampleEntry1 = index._storage.addSmdaReport(self.smda_report_1)
        # id1 = sampleEntry1.sample_id
        sampleEntry2 = index._storage.addSmdaReport(self.smda_report_2)
        id2 = sampleEntry2.sample_id

        sampleEntry1 = index._storage.addSmdaReport(self.smda_report_1)
        id1 = sampleEntry1.sample_id
        index._storage.deleteSample(id1)
        # TODO check if storage.deleteSample is broken

        libraryEntry = index._storage.addSmdaReport(self.library_report)
        id_lib = libraryEntry.sample_id

        sampleEntry3 = index._storage.addSmdaReport(self.smda_report_3)
        id3 = sampleEntry3.sample_id

        # function_entries_1 = index._storage.getFunctionsBySampleId(id1)
        # index.updateMinHashes(function_entries=function_entries_1)

        function_entries_2 = index._storage.getFunctionsBySampleId(id2)
        assert function_entries_2
        worker.updateMinHashesForSample(id2)
        worker.updateMinHashesForSample(id_lib)
        worker.updateMinHashesForSample(id3)

        matcher = MatcherQuery(worker)
        result = matcher.getMatchesForSmdaReport(self.smda_report_1)

        function_matches_expected = deepcopy(self.function_matches_expected)

        for function_data in function_matches_expected:
            matches = function_data["matches"]
            own_function_id = function_data["fid"]
            function_data["fid"] = len(function_entries_2) - own_function_id - 1

        sample_info_expected = SampleEntry(self.smda_report_1, sample_id=-1).toDict()

        self.assertEqual(result["info"]["sample"], sample_info_expected)
        self.assertTrue("duration" in result["info"]["job"])
        self.assertNotEqual(result["info"]["job"]["duration"], 0)
        self.assertTrue("timestamp" in result["info"]["job"])
        self.assertTrue(isinstance(result["info"]["job"]["timestamp"], str))
        self.assertTrue(len(result["info"]["job"]["timestamp"]) > 0)

        self.assertEqual(result["matches"]["aggregation"]["pichash"], self.pichash_aggregation_expected_query)
        self.assertEqual(result["matches"]["aggregation"]["minhash"], self.minhash_aggregation_expected_query)
        json.dumps(sorted(result["matches"]["functions"], key=lambda x: x["fid"]), indent=1)
        self.assertEqual(
            sorted(result["matches"]["functions"], key=lambda x: x["fid"]),
            sorted(function_matches_expected, key=lambda x: x["fid"]),
        )
        self.assertEqual(
            result["matches"]["samples"],
            [
                self.sample_summary_lib_entry_expected,
                self.sample_summary_entry_2_expected,
                self.sample_summary_entry_3_expected,
            ],
        )


if __name__ == "__main__":
    unittest.main()



import os
import re
import json
import hashlib
import argparse

import subprocess

from dotenv import load_dotenv

from smda.Disassembler import Disassembler
from smda.common.SmdaReport import SmdaReport

from mcrit.client.McritClient import McritClient
from mcrit.storage.FamilyEntry import FamilyEntry
from mcrit.storage.SampleEntry import SampleEntry
from mcrit.storage.FunctionEntry import FunctionEntry
from mcrit.storage.MatchingResult import MatchingResult


### Helper functionality for submissions

def is_pe_or_elf(input_path):
    """
    Don't perform a full validation, just check heuristically if the file starts with PE or ELF markings
    """
    data = b""
    with open(input_path, "rb") as fin:
        data = fin.read(4)
    if len(data) >= 4:
        is_elf = data[:4] == b"\x7FELF"
        is_pe = data[:2] == b"MZ"
        return is_elf or is_pe
    return False


def get_base_addr(input_path):
    """
    If provided path ends with 0x[0-9a-fA-F]{8,16}, we assume that this is an IMAGEBASE address. 
    """
    base_addr = None
    baddr_match = re.search(re.compile("0x(?P<base_addr>[0-9a-fA-F]{8,16})$"), input_path)
    if baddr_match:
        base_addr = int(baddr_match.group("base_addr"), 16)
    return base_addr


def sha256(content):
    return hashlib.sha256(content).hexdigest()


def readFileContent(file_path):
    file_content = b""
    with open(file_path, "rb") as fin:
        file_content = fin.read()
    return file_content


def getFamilyName(input_path):
    family_name = ""
    abs_path = os.path.abspath(input_path)
    for folder in abs_path.split("/")[::-1]:
        if folder:
            family_name = folder
    return family_name


def getSampleVersion(input_path, family):
    sample_version = ""
    collected = []
    for folder in input_path.split("/"):
        if folder != family and folder != "":
            collected.append(folder)
    if collected:
        sample_version = os.sep.join(collected)
    return sample_version


def getFolderFilePath(input_root, input_path):
    abs_path = os.path.abspath(input_path)
    relative_filepath = abs_path[len(input_root):]
    return relative_filepath


def getSmdaReportFromFilepath(args, filepath):
    filename = os.path.basename(filepath)
    smda_report = None
    if args.smda:
        if not filename.endswith(".smda"):
            print(f"Skipping a file not recognized as SMDA report: {filepath}")
        else:
            try:
                smda_report = SmdaReport.fromFile(filepath)
            except:
                print(f"Failed to parse SMDA report: {filepath}")
    else:
        if args.executables_only and not is_pe_or_elf(filepath):
            print(f"Skipping a file not recognized as executable: {filepath}")
        else:
            disassembler = Disassembler()
            if get_base_addr(filename) is not None:
                base_addr = get_base_addr(filename)
                try:
                    smda_report = disassembler.disassembleBuffer(readFileContent(filepath), base_addr)
                    smda_report.filename = filename
                except Exception as exc:
                    import traceback
                    print(f"ERROR: SMDA caused an exception while processing this file: {filepath}")
                    print(traceback.format_exc())
                    return None
            else:
                try:
                    smda_report = disassembler.disassembleFile(filepath)
                except:
                    import traceback
                    print(f"ERROR: SMDA caused an exception while processing this file: {filepath}")
                    print(traceback.format_exc())
                    return None
    # apply any of the forced flags: family, version, library
    if smda_report:
        if args.mode in ["file", "dir"]:
            if args.family is not None:
                smda_report.family = args.family
            if args.version is not None:
                smda_report.version = args.version
        if args.mode in ["file", "dir", "recursive"] and args.library:
            smda_report.is_library = True
        if args.output:
            with open() as f_smda:
                json.dump(smda_report.toDict(), f_smda, sort_keys=True, indent=1)
    return smda_report

def submitViaSubprocess(args, filepath):
    command = ["python", "-m", "mcrit", "client", "submit"]
    if args.server:
        command.extend(["--server", args.server])
    if args.apitoken:
        command.extend(["--apitoken", args.apitoken])
    if args.family:
        command.extend(["--family", args.family])
    if args.version:
        command.extend(["--version", args.version])
    if args.library:
        command.extend(["--library"])
    if args.executables_only:
        command.extend(["--executables_only"])
    if args.output:
        command.extend(["--output", args.output])
    if args.smda:
        command.extend(["--smda"])
    command.append(filepath)
    console_handle = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    try:
        stdout_result, stderr_result = console_handle.communicate(timeout=args.worker_timeout)
        stdout_result = stdout_result.strip().decode("utf-8")
        if stdout_result:
            print("STDOUT logs from subprocess: ", stdout_result)
        if stderr_result:
            stderr_result = stderr_result.strip().decode("utf-8")
            print("STDERR logs from subprocess: ", stderr_result)
    except subprocess.TimeoutExpired:
        print(f"Processing {str(filepath)} with a spawned worker timed out during processing.")


### Main Processing 

class McritConsole(object):

    def __init__(self) -> None:
        self.parser = argparse.ArgumentParser(description="Console interaction with MCRIT.")
        subparsers = self.parser.add_subparsers(dest="command")
        self.parser_client = subparsers.add_parser("client")
        subparser_client = self.parser_client.add_subparsers(dest="client_command")
        self.parser_client.add_argument("--server", type=str, default=None, help="The MCRIT server to connect to (overrides dotnev/env).")
        self.parser_client.add_argument("--apitoken", type=str, default=None, help="API token to use for the connection (overrides dotnev/env).")
        client_status = subparser_client.add_parser("status")
        # client submit
        client_submit = subparser_client.add_parser("submit", help="Various ways of file submission incl. disassembly using SMDA if needed.")
        client_submit.add_argument("filepath", type=str, help="Submit the folllowing <filepath>, indicating a (file/dir).")
        client_submit.add_argument("--mode", type=str, default="file", choices=["file", "dir", "recursive", "malpedia"], help="Submit a single <file> or all files in a <dir>. Use <recursive> submission for a folder structured as ./family_name/version/version/files. Synchronize <malpedia> into MCRIT. Default: <file>.")
        client_submit.add_argument("-f", "--family", type=str, help="Set/Override SmdaReport with this family (only in modes: file/dir)")
        client_submit.add_argument("-v", "--version", type=str, help="Set/Override SmdaReport with this version (only in modes: file/dir)")
        client_submit.add_argument("-l", "--library", action="store_true", help="Set/Override SmdaReport with the library flag (only in modes: file/dir/recursive, default: False).")
        client_submit.add_argument("-x", "--executables_only", action="store_true", help="Only process files that are parsable PE or ELF files (default: False).")
        client_submit.add_argument("-o", "--output", type=str, help="Optionally store SMDA reports in folder OUTPUT.")
        client_submit.add_argument("-s", "--smda", action="store_true", help="Do not disassemble, instead only submit files that are recognized as SMDA reports (only works with modes: file/dir).")
        client_submit.add_argument("-w", "--worker", action="store_true", help="Spawn workers to process the submission (only in modes: dir/recursive/malpedia, default: False).")
        client_submit.add_argument("-t", "--worker-timeout", type=int, default=300, help="Timeout for workers to conclude the submission (default: 300 seconds).")
        # client query
        client_query = subparser_client.add_parser("query", help="Query MCRIT with a sample incl. disassembly using SMDA if needed.")
        client_query.add_argument("filepath", type=str, help="Submit the folllowing <filepath>.")
        client_query.add_argument("-a", "--base_addr", type=str, help="Set a base_addr and treat this file as mapped buffer (0x<addr> as hexadecimal int or <addr> as decimal int.")
        client_query.add_argument("-b", "--bitness", type=str, help="When processing as buffer, use this bitness ([32, 64] - default: 32 bit)")
        client_query.add_argument("-o", "--output", type=str, help="Optionally store matching report in folder OUTPUT.")
        client_query.add_argument("-s", "--smda", action="store_true", help="Assume provided input file is a SMDA report.")
        # client import / export / search 
        client_import = subparser_client.add_parser("import", help="Import of previously exported data in the MCRIT format.")
        client_import.add_argument("filepath", type=str, help="Import a given <filepath> containing MCRIT data into the storage.")
        client_export = subparser_client.add_parser("export", help="Export of data from MCRIT.")
        client_export.add_argument("filepath", type=str, help="Export the full storage into <filepath>.")
        client_export.add_argument("--sample_ids", type=str, help="Limit export to a list of comma-separated <sample_ids>.")
        client_search = subparser_client.add_parser("search", help="Search for arbitrary data based on a search string.")
        client_search.add_argument("search_term", type=str, help="Conduct a search with a given <search_term>.")
        client_queue = subparser_client.add_parser("queue", help="Inspection of the current processing queue.")
        client_queue.add_argument("--filter", type=str, help="Filter queue entries with this term.")

    def run(self):
        ARGS = self.parser.parse_args()
        # try to load .env file first
        THIS_FILE_PATH = str(os.path.abspath(__file__))
        PROJECT_ROOT = str(os.path.abspath(os.sep.join([THIS_FILE_PATH, "..", "..", ".."])))
        env_path = os.path.join(PROJECT_ROOT, '.env')
        if os.path.exists(env_path):
            load_dotenv(env_path)
        # regardless of outcome for dotenv, try to load from env
        server = os.environ.get('MCRIT_CLI_SERVER')
        apitoken = os.environ.get('MCRIT_CLI_APITOKEN')
        # always override with command line arguments
        if ARGS.server:
            server = ARGS.server
        if ARGS.apitoken:
            apitoken = ARGS.apitoken
        self.client = McritClient(server, apitoken)
        if ARGS.client_command == "status":
            self._handle_status(ARGS)
        elif ARGS.client_command == "import":
            self._handle_import(ARGS)
        elif ARGS.client_command == "export":
            self._handle_export(ARGS)
        elif ARGS.client_command == "search":
            self._handle_search(ARGS)
        elif ARGS.client_command == "queue":
            self._handle_queue(ARGS)
        elif ARGS.client_command == "query":
            self._handle_query(ARGS)
        elif ARGS.client_command == "submit":
            self._handle_submit(ARGS)
        elif ARGS.client_command == "sync":
            self._handle_sync(ARGS)
        else:
            print(self.parser_client.print_help())

    def _handle_status(self, args):
        result = self.client.getStatus(with_pichash=False)
        if result:
            print(f"DB:        {result['status']['storage_type']} - {result['status']['db_state']} | {result['status']['db_timestamp']}")
            print(f"Families:  {result['status']['num_families']}")
            print(f"Samples:   {result['status']['num_samples']}")
            print(f"Functions: {result['status']['num_functions']}")
        else:
            print("Failed to retrieve status from the server.")

    def _handle_import(self, args):
        if os.path.isfile(args.filepath):
            with open(args.filepath) as fin:
                result = self.client.addImportData(json.load(fin))
                print(result)
        else:
            print("Your <filepath> does not exist.")

    def _handle_export(self, args):
        sample_ids = None
        if args.sample_ids is not None:
            sample_ids = [int(i) for i in args.sample_ids.split(",")]
        result = self.client.getExportData(sample_ids=sample_ids)
        with open(args.filepath, "w") as fout:
            json.dump(result, fout, indent=1)
        print(f"wrote export to {args.filepath}.")

    def _handle_search(self, args):
        result = self.client.search_families(args.search_term)
        if result["search_results"]:
            print("Family Search Results")
            for family_id, entry in result["search_results"].items():
                family_entry = FamilyEntry.fromDict(entry)
                print(f"{family_entry}")
            print("*" * 20)
        result = self.client.search_samples(args.search_term)
        if result["search_results"]:
            print("Sample Search Results")
            for sample_id, entry in result["search_results"].items():
                sample_entry = SampleEntry.fromDict(entry)
                print(f"{sample_entry}")
            print("*" * 20)
        result = self.client.search_functions(args.search_term)
        if result["search_results"]:
            print("Function Search Results")
            for function_id, entry in result["search_results"].items():
                function_entry = FunctionEntry.fromDict(entry)
                print(f"{function_entry}")

    def _handle_queue(self, args):
        result = self.client.getQueueData(filter=args.filter)
        for entry in result:
            job_id = entry.job_id
            result_id = entry.result
            created = entry.created_at if entry.created_at is not None else "-"
            started = entry.started_at if entry.started_at is not None else "-"
            finished = entry.finished_at if entry.finished_at is not None else "-"
            method = entry.parameters
            progress = entry.progress
            print(f"{job_id} {result_id} | {created} {started} {finished} | {method} - {progress}")

    def _handle_query(self, args):
        # run a number of sanity checks first
        if not os.path.exists(args.filepath):
            print("Your <filepath> does not exist.")
            return
        base_addr = None
        if args.base_addr is not None:
            try:
                if args.base_addr.startswith("0x"):
                    base_addr = int(args.base_addr, 16)
                else:
                    base_addr = int(args.base_addr)
            except:
                print("base_addr has invalid format.")
                return
        bitness = 32
        if args.bitness is not None and not args.bitness in ["32", "64"]:
            print("Invalid value for bitness provided.")
            return
        elif args.bitness is not None:
            bitness = int(args.bitness)
        if args.output is not None and (not os.path.exists(args.output) or not os.path.isdir(args.output)):
            print("Your <output> is not a directory or does not exist.")
            return
        if args.smda:
            smda_report = SmdaReport.fromFile(args.filepath)
            job_id = self.client.requestMatchesForSmdaReport(smda_report)
        else:
            if base_addr:
                job_id = self.client.requestMatchesForMappedBinary(readFileContent(args.filepath), disassemble_locally=False, base_address=base_addr)
            else:
                job_id = self.client.requestMatchesForUnmappedBinary(readFileContent(args.filepath), disassemble_locally=False)
        print(f"Started job: {job_id}, waiting for result...")
        compact_result_dict = self.client.awaitResult(job_id, sleep_time=2, compact=True)
        if args.output is not None:
            with open(args.output + os.sep + f"{job_id}.json", "w") as fout:
                json.dump(compact_result_dict, fout, indent=1)
        result = MatchingResult.fromDict(compact_result_dict)
        print(f"{'Family':>30} | {'Version':>20} | {'Sample':>5} | {'SHA256':>8} | {'Func':>5} | {'Min':>5} | {'Pic':>5} | {'Lib':>5} | {'Direct':>13} | {'Freq':>13} | ")
        for family_result in result.getBestSampleMatchesPerFamily(limit=20, malware_only=True):
            result_line = f"{family_result.family:>30} | "
            result_line += f"{family_result.version:>20} | "
            result_line += f"{family_result.sample_id:>6} | "
            result_line += f"{family_result.sha256[:8]} | "
            result_line += f"{family_result.num_functions:>5} | "
            result_line += f"{family_result.matched_functions_minhash:>5} | "
            result_line += f"{family_result.matched_functions_pichash:>5} | "
            result_line += f"{family_result.matched_functions_library:>5} | "
            result_line += f"{family_result.matched_percent_score_weighted:>6.2f} {family_result.matched_percent_nonlib_score_weighted:>6.2f} | "
            result_line += f"{family_result.matched_percent_frequency_weighted:>6.2f} {family_result.matched_percent_nonlib_frequency_weighted:>6.2f} | "
            print(result_line)

    def _handle_submit(self, args):
        # run a number of sanity checks first
        if not os.path.exists(args.filepath):
            print("Your <filepath> does not exist.")
            return
        if args.mode == "file" and not os.path.isfile(args.filepath):
            print("Mode <file> only works when <filepath> is a file.")
            return
        if args.mode in ["dir", "recursive", "malpedia"] and not os.path.isdir(args.filepath):
            print("Modes <dir|recursive|malpedia> only work when <filepath> is a directory.")
            return
        if args.output is not None and (not os.path.exists(args.output) or not os.path.isdir(args.output)):
            print("Your <output> is not a directory or does not exist.")
            return
        if args.smda and args.mode in ["recursive", "malpedia"]:
            print("Modes <recursive|malpedia> are not compatible with SMDA report loading.")
            return
        if args.worker and args.mode not in ["dir", "recursive", "malpedia"]:
            print("Mode <worker> only works with modes: dir/recursive/malpedia.")
            return
        # behavior according to the modes offered
        if args.mode == "file":
            self._handle_submit_file(args)
        if args.mode == "dir":
            self._handle_submit_dir(args)
        if args.mode == "recursive":
            self._handle_submit_recursive(args)
        if args.mode == "malpedia":
            self._handle_submit_malpedia(args)

    def _handle_submit_file(self, args):
        sample_sha256 = sha256(readFileContent(args.filepath))
        if self.client.getSampleBySha256(sample_sha256):
            print(f"SKIPPING: {args.filepath} - already in MCRIT.")
            return
        smda_report = getSmdaReportFromFilepath(args, args.filepath)
        if smda_report:
            print(smda_report)
            self.client.addReport(smda_report)

    def _handle_submit_dir(self, args):
        mcrit_samples = self.client.getSamples()
        mcrit_samples_by_sha256 = {}
        for sample_id, sample in mcrit_samples.items():
            mcrit_samples_by_sha256[sample.sha256] = sample
        for filename in os.listdir(args.filepath):
            filepath = os.sep.join([args.filepath, filename])
            if os.path.isfile(filepath):
                if sha256(readFileContent(filepath)) in mcrit_samples_by_sha256:
                    print(f"SKIPPING: {filepath} - already in MCRIT.")
                    continue
                if args.worker:
                    submitViaSubprocess(args, filepath)
                else:
                    smda_report = getSmdaReportFromFilepath(args, filepath)
                    if smda_report:
                        print(smda_report)
                        self.client.addReport(smda_report)

    def _handle_submit_recursive(self, args):
        mcrit_samples = self.client.getSamples()
        mcrit_samples_by_sha256 = {}
        for sample_id, sample in mcrit_samples.items():
            mcrit_samples_by_sha256[sample.sha256] = sample
        for root, subdir, files in sorted(os.walk(args.filepath)):
            folder_relative_path = getFolderFilePath(args.filepath, root)
            for filename in files:
                filepath = os.sep.join([root, filename])
                if os.path.isfile(filepath):
                    if sha256(readFileContent(filepath)) in mcrit_samples_by_sha256:
                        print(f"SKIPPING: {filepath} - already in MCRIT.")
                        continue
                    if args.worker:
                        args.family = getFamilyName(folder_relative_path)
                        args.version = getSampleVersion(folder_relative_path, args.family)
                        submitViaSubprocess(args, filepath)
                    else:
                        smda_report = getSmdaReportFromFilepath(args, filepath)
                        if smda_report:
                            smda_report.family = getFamilyName(folder_relative_path)
                            smda_report.version = getSampleVersion(folder_relative_path, smda_report.family)
                            print(filepath)
                            print(smda_report)
                            self.client.addReport(smda_report)

    def _handle_submit_malpedia(self, args):
        # verify that we have a malpedia root
        malpedia_root = os.path.abspath(args.filepath)
        malpedia_root = malpedia_root.rstrip("/")
        if not malpedia_root.endswith("malpedia"):
            print(f"Error: You pointing to a folder named differently than 'malpedia'.")
            return
        if not "malpedia.bib" in os.listdir(malpedia_root):
            print(f"Error: 'malpedia.bib' is missing in that folder, are you sure you are poniting to a Malpedia repository?")
            return
        # get current status of all samples in MCRIT
        mcrit_samples = self.client.getSamples()
        mcrit_samples_by_filename = {}
        for sample_id, sample in mcrit_samples.items():
            # verify that filename has malpedia format (starts with sha256 and _unpacked/_dump)
            if sample.filename in mcrit_samples_by_filename:
                print(f"WARNING: filename {sample.filename} appears to exist more than once in your MCRIT instance, now using SHA256 {sample.sha256[:8]}.")
            mcrit_samples_by_filename[sample.filename] = sample
        # get all unpacked/dumped files and their family/version by crawling given Malpedia location
        malpedia_samples_by_filename = self._getMalpediaSamplesByFilename(malpedia_root)
        # use submission filenames within MCRIT to check for presence of files and verify their family/version identity
        for filename, malpedia_info in malpedia_samples_by_filename.items():
            malpedia_family = malpedia_info["family"]
            malpedia_version = malpedia_info["version"]
            malpedia_filepath = malpedia_info["filepath"]
            if filename in mcrit_samples_by_filename:
                mcrit_family = mcrit_samples_by_filename[filename].family
                mcrit_version = mcrit_samples_by_filename[filename].version
                # warn about files were family/version mismatches.
                if (malpedia_family != mcrit_family or
                        malpedia_version != mcrit_version):
                    print(f"WARNING: Sample {mcrit_samples_by_filename[filename].sample_id} with filename {filename} has different family/version information: ")
                    print(f"* Malpedia: {malpedia_family}|{malpedia_version}")
                    print(f"* MCRIT: {mcrit_family}|{mcrit_version}")
            # directly add all files in Malpedia but missing in MCRIT
            else:
                if args.worker:
                    args.family = malpedia_family
                    args.version = malpedia_version
                    submitViaSubprocess(args, malpedia_filepath)
                else:
                    smda_report = getSmdaReportFromFilepath(args, malpedia_filepath)
                    if smda_report:
                        smda_report.family = malpedia_family
                        smda_report.version = malpedia_version
                        print(malpedia_filepath)
                        print(smda_report)
                        self.client.addReport(smda_report)
        # warn about files that appear deleted because not present in Malpedia but in MCRIT (based on name schema)
        for filename, mcrit_sample in mcrit_samples_by_filename.items():
            if self._isMalpediaFilename(filename) and filename not in malpedia_samples_by_filename:
                print(f"WARNING: Sample {mcrit_sample.sample_id} with filename {filename} ({mcrit_sample.family}|{mcrit_sample.version}) present in MCRIT but not in Malpedia?")

    def _getMalpediaSamplesByFilename(self, malpedia_root):
        malpedia_samples_by_filename = {}
        for root, subdir, files in sorted(os.walk(malpedia_root)):
            if not "win." in root or "elf." in root:
                continue
            folder_relative_path = getFolderFilePath(malpedia_root, root)
            for filename in sorted(files):
                if self._isMalpediaFilename(filename):
                    filepath = root + os.sep + filename
                    sample_family = getFamilyName(folder_relative_path)
                    malpedia_samples_by_filename[filename] = {
                        "filename": filename,
                        "filepath": filepath,
                        "family": sample_family,
                        "version": getSampleVersion(folder_relative_path, sample_family)
                    }
        return malpedia_samples_by_filename
    
    def _isMalpediaFilename(self, filename):
        malpedia_file_pattern = re.compile("^[0-9a-f]{64}(_unpacked|dump7?_0x[0-9a-fA-F]{8,16})")
        return re.search(malpedia_file_pattern, filename)



#!/usr/bin/env python3

import os
import re
import uuid
import json
import time
import logging
import hashlib
from random import sample
from datetime import datetime, timedelta
from collections import defaultdict
from itertools import zip_longest
from multiprocessing import Pool, cpu_count
from typing import Dict, List, Optional, TYPE_CHECKING, Tuple

import tqdm
from smda.common.BinaryInfo import BinaryInfo
from smda.common.SmdaFunction import SmdaFunction
from smda.common.SmdaReport import SmdaReport
from smda.Disassembler import Disassembler
from smda.SmdaConfig import SmdaConfig
from smda.common.SmdaInstruction import SmdaInstruction
from smda.intel.IntelInstructionEscaper import IntelInstructionEscaper

from mcrit.Worker import Worker
from mcrit.config.McritConfig import McritConfig
from mcrit.config.MinHashConfig import MinHashConfig
from mcrit.config.ShinglerConfig import ShinglerConfig
from mcrit.config.QueueConfig import QueueConfig
from mcrit.config.McritConfig import McritConfig
from mcrit.config.StorageConfig import StorageConfig
from mcrit.matchers.MatcherCross import MatcherCross
from mcrit.matchers.MatcherQuery import MatcherQuery
from mcrit.matchers.MatcherSample import MatcherSample
from mcrit.matchers.MatcherVs import MatcherVs
from mcrit.minhash.MinHasher import MinHasher
from mcrit.queue.LocalQueue import Job
from mcrit.queue.QueueFactory import QueueFactory
from mcrit.queue.QueueRemoteCalls import NoProgressReporter, QueueRemoteCallee, Remote, JobProgressReporter
from mcrit.storage.SampleEntry import SampleEntry
from mcrit.storage.StorageFactory import StorageFactory

if TYPE_CHECKING:
    from mcrit.storage.StorageInterface import StorageInterface

logging.basicConfig(level=logging.INFO)
LOGGER = logging.getLogger(__name__)


class SingleJobWorker(Worker):
    def __init__(self, job_id, queue=None, config=None, storage: Optional["StorageInterface"] = None, profiling=False):
        self.job_id = job_id
        self._worker_id = f"Worker-{uuid.uuid4()}"
        LOGGER.info(f"Starting as worker: {self._worker_id}")
        if config is None:
            config = McritConfig()

        if not queue:
            queue = QueueFactory().getQueue(config, consumer_id=self._worker_id)

        if profiling:
            print("[!] Running as profiled application.")
            profiling_path = os.path.abspath(os.path.join(os.path.dirname(os.path.abspath(__file__ )), "..", "profiler"))
            os.makedirs(profiling_path, exist_ok=True)
        else:
            profiling_path = None
        super().__init__(queue=queue, config=config, storage=storage, profiling=profiling)

        self.config = config
        self._storage_config = config.STORAGE_CONFIG
        self._minhash_config = config.MINHASH_CONFIG
        self._shingler_config = config.SHINGLER_CONFIG
        self._queue_config = config.QUEUE_CONFIG
        self.minhasher = MinHasher(config.MINHASH_CONFIG, config.SHINGLER_CONFIG)
        if storage:
            self._storage = storage
        else:
            self._storage = StorageFactory.getStorage(config)

    def  __enter__(self):
        return self

    def  __exit__(self, *args):
        # TODO unregister our worker_id from all in-progress jobs found in the queue
        self.queue.unregisterWorker()
        self.queue.release_all_jobs()

    #### Overwrite inherited methods to achive execution of a single job ####
        
    def _executeJobPayload(self, job_payload, job):
        LOGGER.debug("DECODE JOB: %s", job_payload)
        method, params, kwparams = self._decodeJobPayload(job_payload)
        # Add progress reporter if necessary:
        if method.progressor:
            LOGGER.debug("kwparams: %s", kwparams)
            kwparams["progress_reporter"] = JobProgressReporter(job, 0.1)
        LOGGER.debug("EXECUTE JOB: %s", job_payload)
        result = method(*params, **kwparams)
        LOGGER.debug("FINISHED JOB: %s", job_payload)
        return result

    def _executeJob(self, job):
        try:
            with job as j:
                LOGGER.info("Processing Remote Job: %s", job)
                result = self._executeJobPayload(j["payload"], job)
                LOGGER.debug("Remote Job Result: %s", result)
                # ensure we always have a job_id for finished job payloads
                result_id = self.queue._dicts_to_grid(result, metadata={"result": True, "job": job.job_id})
                print(str(result_id))
                job.result = result_id
                LOGGER.info("Finished Remote Job producing result_id: %s", result_id)
        except Exception as exc:
            pass

    def run(self):
        self._alive = True
        job = self.queue.get_job(self.job_id)
        LOGGER.debug("Found job")
        self._executeJob(job)

    def terminate(self):
        self._alive = False



#!/usr/bin/python

import json
import logging
import os
import unittest
from copy import deepcopy

from smda.common.SmdaReport import SmdaReport

from mcrit.minhash.MinHash import MinHash
from mcrit.storage.FunctionEntry import FunctionEntry
from mcrit.storage.SampleEntry import SampleEntry
from mcrit.storage.MatchingResult import MatchingResult

from .context import config

LOG = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO, format="%(asctime)-15s %(message)s")
logging.disable(logging.CRITICAL)


class MinHashingTestSuite(unittest.TestCase):
    """Run a full example on a memory dump"""

    def testFunctionEntry(self):
        THIS_FILE_PATH = str(os.path.abspath(__file__))
        PROJECT_ROOT = str(os.path.abspath(os.sep.join([THIS_FILE_PATH, "..", ".."])))
        example_file_path = os.sep.join([PROJECT_ROOT, "tests", "example_report.smda"])
        with open(example_file_path, "r") as fjson:
            smda_json = json.load(fjson)

        smda_report = SmdaReport.fromDict(smda_json)
        sample_entry = SampleEntry(smda_report, sample_id=0, family_id=0)
        minhash = MinHash(function_id=1, minhash_signature=[0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39])
        for smda_function in smda_report.getFunctions():
            function_entry = FunctionEntry(sample_entry, smda_function, 0, minhash=minhash)

            as_dict = function_entry.toDict()
            as_entry = FunctionEntry.fromDict(as_dict)

            # test text-friendly encoding of binary pichashes
            self.assertEqual(as_entry.minhash, function_entry.minhash)

    def testSampleEntry(self):
        THIS_FILE_PATH = str(os.path.abspath(__file__))
        PROJECT_ROOT = str(os.path.abspath(os.sep.join([THIS_FILE_PATH, "..", ".."])))
        example_file_path = os.sep.join([PROJECT_ROOT, "tests", "example_report.smda"])
        with open(example_file_path, "r") as fjson:
            smda_json = json.load(fjson)

        smda_report = SmdaReport.fromDict(smda_json)
        sample_entry = SampleEntry(smda_report, sample_id=0, family_id=0)

        as_dict = sample_entry.toDict()
        as_entry = SampleEntry.fromDict(as_dict)
        self.assertEqual(as_entry.sample_id, sample_entry.sample_id)

    def testMatchingResult(self):
        THIS_FILE_PATH = str(os.path.abspath(__file__))
        PROJECT_ROOT = str(os.path.abspath(os.sep.join([THIS_FILE_PATH, "..", ".."])))
        example_file_path = os.sep.join([PROJECT_ROOT, "tests", "example_matching_report.json"])
        with open(example_file_path, "r") as fjson:
            match_json = json.load(fjson)
        matching_result = MatchingResult.fromDict(match_json)
        assert len(matching_result.sample_matches) == 2
        assert len(matching_result.getFunctionMatches()) == 719
        assert len(set([match.function_id for match in matching_result.getFunctionMatches()])) == 515
        ### test filtering
        # test filtering by family and sample counts
        filtered_result = deepcopy(matching_result)
        filtered_result.filterToFamilyCount(1)
        assert len(filtered_result.getFunctionMatches()) == 414
        assert len(filtered_result.getFunctionMatches(unfiltered=True)) == 719
        filtered_result = deepcopy(matching_result)
        filtered_result.filterToSampleCount(max_samples=1)
        assert len(filtered_result.getFunctionMatches()) == 414
        filtered_result.filterToSampleCount(min_samples=2)
        assert len(filtered_result.getFunctionMatches()) == 0
        # filter by score / library
        filtered_result = deepcopy(matching_result)
        filtered_result.filterToFunctionScore(95)
        assert len(filtered_result.getFunctionMatches()) == 581
        filtered_result = deepcopy(matching_result)
        filtered_result.filterToFunctionScore(min_score=95, library_only=True)
        assert len(filtered_result.getFunctionMatches()) == 718
        filtered_result = deepcopy(matching_result)
        filtered_result.excludeLibraryMatches()
        assert len(filtered_result.getFunctionMatches()) == 715
        assert len(set([match.function_id for match in filtered_result.getFunctionMatches()])) == 513


if __name__ == "__main__":
    unittest.main()



import logging
import os
import time
from threading import Thread
from unittest import TestCase

import pymongo
from bson.json_util import dumps, loads
from mcrit.config.QueueConfig import QueueConfig
from mcrit.libs.mongoqueue import MongoQueue
from mcrit.queue.LocalQueue import LocalQueue
from mcrit.queue.QueueRemoteCalls import (
    NoProgressReporter,
    QueueRemoteCallee,
    QueueRemoteCaller,
    Remote,
    _createJobPayload,
    get_descriptor,
    upload_file_params,
)
#For test marks
import pytest

from .context import config

LOG = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO, format="%(asctime)-15s %(message)s")
logging.disable(logging.CRITICAL)


class myWorker(QueueRemoteCallee):
    @Remote(
        file_locations=[0, 2, 1],
        json_locations=[3, 4],
        kwfile_locations=["eggs"],
        kwjson_locations=["foo"],
    )
    def test(self, *params, **kwparams):
        if "force_recalculation" in kwparams:
            raise AttributeError
        return {"args": dumps(params, sort_keys=True), "kwargs": dumps(kwparams, sort_keys=True)}

    @Remote(progress=True)
    def test_progress(self, progress_reporter=NoProgressReporter()):
        # if not progress_reporter:
        #    progress_reporter = NoProgressReporter()
        l = 100

        # raise Exception("The job received a terminatation message while reporting progress")
        progress_reporter._report_interval = 0

        progress_reporter.set_total(l)
        for i in range(l):
            print(l)
            time.sleep(0.01)
            progress_reporter.step()
        return 0x42

    def function_that_isnt_remote(self, a, b, c):
        pass

    @Remote()
    def child_job(self, i):
        time.sleep(0.2)
        return i*i
    
    @Remote()
    def parent_job(self, child_jobs):
        results = []
        for id in child_jobs:
            results.append(self.getResultForJob(id))
        results = [i for i in results if i is not None]
        return sum(results)

    

# Define Caller Class
class Caller(QueueRemoteCaller(myWorker)):
    def start_child_and_parent_job(self):
        child_job_ids = []
        for i in range(9):
            child_job_ids.append(self.child_job(i, force_recalculation=True))
        # await_jobs param is managed by queue and not passed to function.
        # Maybe call it job_dependencies?
        # parent job will only be executed after all await_jobs were finished, terminated or failed.
        parent_job_id = self.parent_job(child_job_ids, await_jobs=child_job_ids)
        return parent_job_id	



class RemoteCalleeTest(TestCase):
    def setUp(self):
        self.queue = LocalQueue()
        self.worker = myWorker(self.queue)

    def test_progress_local(self):
        self.worker.test_progress()


class LocalQueueRemoteCallTest(TestCase):
    def setUp(self):
        self.queue = LocalQueue()
        self.worker = myWorker(self.queue)
        self.queue.set_worker(self.worker)
        self.caller = Caller(self.queue)
        self.queue.clean_interval = 1e100

    # TODO do a real test here
    @pytest.mark.sleep
    def test_progress(self):
        id = self.caller.test_progress()
        time.sleep(0.2)
        job = self.queue.get_job(id)
        self.assertGreater(job._data["progress"], 0)

    @pytest.mark.sleep
    def test_termination(self):
        id = self.caller.test_progress()
        job = self.queue.get_job(id)
        status_termination = job.terminate()
        print("termination:", status_termination)
        print()
        time.sleep(0.1)
        self.assertTrue(job._is_terminated())

    def test_remote_calls(self):
        params_list = [[], [b"bim", b"bam", b"bum", ["hi", {"ha": "ho"}], {"answer": 42}, {"universe": "exists"}]]
        kwparams_list = [{}, dict(foo={"bar": False}, spam={19: "hi"}, orange=[2, 3, {5: "deep thought"}], eggs=b"bar")]
        for params in params_list:
            for kwparams in kwparams_list:
                job_id = self.caller.test(*params, **kwparams)
                result_id = self.caller.awaitResult(job_id)
                print("RID", result_id, job_id)
                result = self.queue._grid_to_dicts(result_id)
                self.assertEqual(result, self.worker.test(*params, **kwparams))
                self.assertEqual(str(self.queue._grid_to_meta(result_id)["job"]), job_id)
        self.queue.clear()

    @pytest.mark.sleep
    def test_clean_period(self):
        old_cache_time = self.queue.cache_time
        old_clean_interval = self.queue.clean_interval
        self.queue.cache_time = 0.0
        self.queue.clean()
        self.queue.clean_interval = 0.8

        params = []
        kwparams = {"foo": {"test_clean_period": True}}

        time.sleep(self.queue.clean_interval * 1.2)
        job_id_1 = self.caller.test(*params, force_recalculation=True, **kwparams)
        self.caller.awaitResult(job_id_1)

        job_id_2 = self.caller.test(*params, force_recalculation=True, **kwparams)
        self.caller.awaitResult(job_id_2)

        self.assertNotEqual(self.queue.get_job(job_id_1), None)

        time.sleep(self.queue.clean_interval * 1.2)
        job_id_1 = self.caller.test(*params, force_recalculation=True, **kwparams)
        self.caller.awaitResult(job_id_1)

        time.sleep(self.queue.clean_interval * 1.2)

        job_id_2 = self.caller.test(*params, force_recalculation=True, **kwparams)
        self.caller.awaitResult(job_id_2)

        self.assertEqual(self.queue.get_job(job_id_1), None)

        self.queue.cache_time = old_cache_time
        self.queue.clean_interval = old_clean_interval

    def test_multiple_cleans(self):
        params = [b"cde"]
        kwparams = {"test_queue_clean": True, "foo": "abc"}

        # start job 2 times
        job_id = self.caller.test(*params, force_recalculation=True, **kwparams)
        self.caller.awaitResult(job_id)

        job_id_2 = self.caller.test(*params, force_recalculation=True, **kwparams)
        self.caller.awaitResult(job_id_2)

        self.queue.clean()
        self.queue.clean()
        self.queue.clean()

    def test_file_cache(self):
        def get_file_id(job_id):
            payload = self.queue.get_job(job_id).payload
            file_params = loads(payload["file_params"])
            return file_params["foo"]

        params = []
        kwparams = {"foo": {"test_file_cache": True}}
        job_id_1 = self.caller.test(*params, force_recalculation=False, **kwparams)
        job_id_2 = self.caller.test(*params, force_recalculation=True, **kwparams)

        file_1 = get_file_id(job_id_1)
        file_2 = get_file_id(job_id_2)

        self.assertEqual(file_1, file_2)

    def test_job_cache(self):
        params_list = [[], [b"bim", b"bam", b"bum", ["hi", {"ha": "ho"}], {"answer": 42}, {"universe": "exists"}]]
        kwparams_list = [{}, dict(foo={"bar": False}, spam={19: "hi"}, orange=[2, 3, {5: "deep mind"}], eggs=b"bar")]
        for params in params_list:
            for kwparams in kwparams_list:
                job_id_1 = self.caller.test(*params, **kwparams)
                job_id_2 = self.caller.test(*params, **kwparams)
                self.assertEqual(job_id_1, job_id_2)
        self.queue.clear()
        for params in params_list:
            for kwparams in kwparams_list:
                job_id_1 = self.caller.test(*params, **kwparams)
                job_id_2 = self.caller.test(*params, force_recalculation=True, **kwparams)
                self.assertNotEqual(job_id_1, job_id_2)
        self.queue.clear()

    def test_function_access(self):
        params = {0: 1, 1: 2, 2: 3}
        payload1 = _createJobPayload(
            "function_that_isnt_remote", params, {}, get_descriptor("function_that_isnt_remote", params, {})
        )
        payload2 = _createJobPayload(
            "function_that_doesnt_exist", params, {}, get_descriptor("function_that_doesnt_exist", params, {})
        )
        job_id = self.queue.put(payload1)
        # self.caller.awaitResult(job_id)
        error_message = self.caller.getJob(job_id).last_error
        # self.assertTrue("NotImplementedError" in error_message)
        # with self.assertRaises(AttributeError):
        #    self.queue.put(payload2)

        # This is also done for mongo
        with self.assertRaises(AttributeError):
            self.caller.function_that_isnt_remote(1, 2, 3)
        with self.assertRaises(AttributeError):
            self.caller.function_that_doesnt_exist(1, 2, 3)

    def test_file_access(self):
        params = []
        kwparams = {"foo": {"test_file_access": True}}
        job_id = self.caller.test(*params, force_recalculation=False, **kwparams)
        payload = self.queue.get_job(job_id).payload
        file_params = loads(payload["file_params"])
        file = file_params["foo"]
        result = self.queue._grid_to_dicts(file)
        self.assertEqual("Access Not Allowed", result)

    def test_file_job_ref(self):
        params = []
        kwparams = {"foo": {"test_file_job_ref": True}}
        job_ids = []
        for i in range(5):
            kwparams["tested_job"] = i
            job_id = self.caller.test(*params, force_recalculation=False, **kwparams)
            job_ids.append(job_id)
            payload = self.queue.get_job(job_id).payload
            file_params = loads(payload["file_params"])
            file = file_params["foo"]
            meta = self.queue._grid_to_meta(file)
            print(meta)
            self.assertEqual(job_ids, meta["jobs"])
            self.assertEqual(0, meta["tmp_lock"])

    def test_file_tmp_lock(self):
        params = []
        kwparams = {"foo": {"test_tmp_lock": True}}
        job_id = self.caller.test(*params, force_recalculation=False, **kwparams)
        payload = self.queue.get_job(job_id).payload
        file_params = loads(payload["file_params"])
        hashes = loads(payload["descriptor"])[2]
        upload_file_params(self.caller, file_params, hashes)
        # lock should be 1 now
        file = file_params["foo"]
        meta = self.queue._grid_to_meta(file)
        self.assertEqual(1, meta["tmp_lock"])

    @pytest.mark.sleep
    def test_queue_clean(self):
        self.queue.cache_time = 0.1
        self.queue.clean_interval = 1e100

        params = [b"cde"]
        kwparams = {"test_queue_clean": True, "foo": "abc"}

        # start job 2 times
        job_id = self.caller.test(*params, force_recalculation=True, **kwparams)
        result_id = self.caller.awaitResult(job_id)

        job_id_2 = self.caller.test(*params, force_recalculation=True, **kwparams)
        result_id_2 = self.caller.awaitResult(job_id_2)

        # extract file ids
        files = loads(self.queue.get_job(job_id).payload["file_params"])

        # start job again but with changed params
        time.sleep(self.queue.cache_time)
        params = []
        job_id_3 = self.caller.test(*params, force_recalculation=True, **kwparams)
        result_id_3 = self.caller.awaitResult(job_id_3)

        # clean. This should impact job 1 and 2, but not 3.
        # we expect parameter 0 to be deleted, but not foo
        self.queue.clean()

        # Check jobs
        self.assertEqual(self.queue.get_job(job_id), None)
        self.assertEqual(self.queue.get_job(job_id_2), None)
        self.assertNotEqual(self.queue.get_job(job_id_3), None)

        # check results
        self.assertEqual(self.queue._grid_to_file(result_id), None)
        self.assertEqual(self.queue._grid_to_file(result_id_2), None)
        self.assertNotEqual(self.queue._grid_to_file(result_id_3), None)

        # Check params
        self.assertNotEqual(self.queue._grid_to_file(files["foo"], results_only=False), None)
        self.assertEqual(self.queue._grid_to_file(files["0"], results_only=False), None)

        # Check foo, if jobs were cleaned:
        self.assertEqual(self.queue._grid_to_meta(files["foo"])["jobs"], [job_id_3])

    def test_job_dependencies(self):
        self.queue.clear()
        additional_worker = myWorker(self.queue)
        additional_worker_thread = Thread(target=additional_worker.run)
        additional_worker_thread.start()

        job_id = self.caller.start_child_and_parent_job()
        result_id = self.caller.awaitResult(job_id)
        result = self.caller.getResult(result_id)
        job = self.caller.getJob(job_id)
        self.assertEqual(result, sum([i*i for i in range(9)]))
        
        additional_worker.terminate()
        self.queue.clear()


### Added mongo attribute
@pytest.mark.mongo
class MongoQueueRemoteCallTest(LocalQueueRemoteCallTest):
    def setUp(self):
        self.client = pymongo.MongoClient(os.environ.get("TEST_MONGODB"))
        queue_config = QueueConfig()
        queue_config.QUEUE_MONGODB_DBNAME = "test_queue_remote_calls"
        queue_config.QUEUE_MONGODB_COLLECTION_NAME = "queue_1"
        self.queue = MongoQueue(queue_config, "consumer_1")
        self.caller = Caller(self.queue)
        self.queue.clean_interval = 1e100
        self.worker = myWorker(self.queue)
        self.worker_thread = Thread(target=self.worker.run)
        self.worker_thread.start()

    @pytest.mark.sleep
    def test_termination(self):
        id = self.caller.test_progress()
        job = self.queue.get_job(id)
        time.sleep(0.1)
        status_termination = job.terminate()
        print("termination:", status_termination)
        print()
        time.sleep(0.5)
        self.assertTrue(job._is_terminated())
        job = self.queue.get_job(id)
        print(job._data)
        self.assertTrue("The job received a terminatation message while reporting progress" in job.last_error)
        result_id = self.caller.awaitResult(id)
        self.assertEqual(result_id, None)

    def test_job_cache_prio(self):
        params = []
        kwparams = {"test_cache_prio": True}

        job_id = self.caller.test(*params, **kwparams)
        self.caller.awaitResult(job_id)

        job_id_2 = self.caller.test(*params, force_recalculation=True, **kwparams)
        self.caller.awaitResult(job_id_2)

        job_id_3 = self.caller.test(*params, force_recalculation=False, **kwparams)
        self.assertEqual(job_id_2, job_id_3)

        self.worker.terminate()
        self.worker_thread.join()
        job_id_4 = self.caller.test(*params, force_recalculation=True, **kwparams)
        job_id_5 = self.caller.test(*params, force_recalculation=False, **kwparams)
        print(self.queue.get_job(job_id_4)._data)
        print(self.queue.get_job(job_id_2)._data)
        self.assertNotEqual(job_id_2, job_id_5)
        self.assertEqual(job_id_4, job_id_5)

        self.worker_thread = Thread(target=self.worker.run)
        self.worker_thread.start()

    def test_function_access(self):
        with self.assertRaises(AttributeError):
            self.caller.function_that_isnt_remote(1, 2, 3)
        with self.assertRaises(AttributeError):
            self.caller.function_that_doesnt_exist(1, 2, 3)

    def tearDown(self):
        self.worker.terminate()
        self.worker_thread.join()
        self.client.drop_database("test_queue_remote_calls")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python

import json
import logging
import os
import unittest

from smda.common.SmdaFunction import SmdaFunction
from smda.common.SmdaReport import SmdaReport

from mcrit.config.McritConfig import McritConfig
from mcrit.config.StorageConfig import StorageConfig
from mcrit.config.ShinglerConfig import ShinglerConfig
from mcrit.minhash.MinHash import MinHash
from mcrit.minhash.MinHasher import MinHasher
from mcrit.minhash.ShingleLoader import ShingleLoader
from mcrit.storage.StorageInterface import StorageInterface

from .context import config

LOG = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO, format="%(asctime)-15s %(message)s")
logging.disable(logging.CRITICAL)


class MinHashingTestSuite(unittest.TestCase):
    """Run a full example on a memory dump"""

    def testMinHash(self):
        THIS_FILE_PATH = str(os.path.abspath(__file__))
        PROJECT_ROOT = str(os.path.abspath(os.sep.join([THIS_FILE_PATH, "..", ".."])))
        example_file_path = os.sep.join([PROJECT_ROOT, "tests", "example_report.smda"])
        smda_report = SmdaReport.fromFile(example_file_path)
        # with open(example_file_path, "r") as fjson:
        #    smda_json = json.load(fjson)
        #    example_functions = [f for key, entry in smda_json["xcfg"].items()]
        shingler_test_config = ShinglerConfig()
        SHINGLER_WEIGHT_STRATEGY = ShingleLoader.WEIGHT_STRATEGY_SHINGLER_WEIGHTS
        SHINGLER_LOGBUCKETS = 100000
        SHINGLER_LOGBUCKET_RANGE = 1
        SHINGLER_LOGBUCKET_CENTERED = True
        SHINGLERS_WEIGHTS = {"FuzzyStatPairShingler": 1, "EscapedBlockShingler": 3}
        SHINGLERS_SEED = 0xDEADBEEF
        SHINGLERS_XOR_VALUES = []
        minhasher = MinHasher(config.MINHASH_CONFIG, shingler_test_config)
        # expected: sequence of (little endian) packed int8/32 values
        expected = {0: bytes.fromhex("7e71b7cd63682ed9"), 4424: bytes.fromhex("d018c68a829734ed")}
        tested = []
        for function in smda_report.getFunctions():  # example_functions:
            minhash = minhasher._calculateMinHash(function)
            print(function.offset, minhash.minhash.hex()[:16])
            if function.offset in expected:
                self.assertTrue(minhash.minhash.startswith(expected[function.offset]))
                tested.append(function.offset)
            if function.offset == 356:
                self.assertFalse(minhasher.isMinHashableFunction(function))
                tested.append(356)
        self.assertTrue(set(expected.keys()) <= set(tested))
        self.assertTrue(356 in tested)

    def testMatching(self):
        minhash_a = MinHash(
            function_id=1, minhash_signature=[0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39]
        )
        minhash_b = MinHash(
            function_id=2, minhash_signature=[0x30, 0x31, 0x30, 0x33, 0x30, 0x30, 0x30, 0x37, 0x38, 0x39]
        )
        minhash_score = MinHash.calculateMinHashScore(minhash_a.minhash, minhash_b.minhash)
        minhash_score_int = MinHash.calculateMinHashIntScore(minhash_a.minhash_int, minhash_b.minhash_int)
        self.assertEqual(60.0, minhash_score)
        self.assertEqual(minhash_score, minhash_score_int)

    def testBandingVariableSize(self):
        config = McritConfig()
        config.STORAGE_CONFIG = StorageConfig()
        config.STORAGE_CONFIG.STORAGE_BANDS = {2: 2, 3: 1}
        config.STORAGE_CONFIG.STORAGE_BAND_SEED = 0
        minhash_a = MinHash(
            function_id=1, minhash_signature=[0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39]
        )
        minhash_b = MinHash(
            function_id=2, minhash_signature=[0x30, 0x31, 0x30, 0x33, 0x30, 0x30, 0x30, 0x37, 0x38, 0x39]
        )

        storage = StorageInterface(config)
        band_hashes_a = storage.getBandHashesForMinHash(minhash_a)
        band_hashes_b = storage.getBandHashesForMinHash(minhash_b)

        self.assertEqual(band_hashes_a[0], band_hashes_b[0])
        self.assertEqual(band_hashes_a[2], band_hashes_b[2])
        self.assertNotEqual(band_hashes_a[1], band_hashes_b[1])

    def testBandingVariableSize8Bits(self):
        config = McritConfig()
        config.STORAGE_CONFIG = StorageConfig()
        config.STORAGE_CONFIG.STORAGE_BANDS = {2: 2, 3: 1}
        config.STORAGE_CONFIG.STORAGE_BAND_SEED = 0
        minhash_a = MinHash(
            function_id=1,
            minhash_signature=[0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39],
            minhash_bits=8,
        )
        minhash_b = MinHash(
            function_id=2,
            minhash_signature=[0x30, 0x31, 0x30, 0x33, 0x30, 0x30, 0x30, 0x37, 0x38, 0x39],
            minhash_bits=8,
        )

        storage = StorageInterface(config)
        band_hashes_a = storage.getBandHashesForMinHash(minhash_a)
        band_hashes_b = storage.getBandHashesForMinHash(minhash_b)

        self.assertEqual(band_hashes_a[0], band_hashes_b[0])
        self.assertEqual(band_hashes_a[2], band_hashes_b[2])
        self.assertNotEqual(band_hashes_a[1], band_hashes_b[1])


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python

import logging
import os
import unittest

from .context import config

LOG = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO, format="%(asctime)-15s %(message)s")
logging.disable(logging.CRITICAL)


class McritTestSuite(unittest.TestCase):
    """Template for test classes"""

    @classmethod
    def setUpClass(cls):
        super(McritTestSuite, cls).setUpClass()
        # perform any global initializations needed for all tests
        cls.tmp = None

    def testSomething(self):
        assert True


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python

import json
import logging
import os
import unittest

from mcrit.libs.utility import generate_unique_pairs, compress_encode, decompress_decode

from .context import config

LOG = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO, format="%(asctime)-15s %(message)s")
logging.disable(logging.CRITICAL)


class TestUtility(unittest.TestCase):
    """Run a full example on a memory dump"""

    def testUniquePairGenerator(self):
        input_data = [1, 2, 3, 4]
        expected_pairs = [(1, 2), (1, 3), (1, 4), (2, 3), (2, 4), (3, 4)]
        pairs = generate_unique_pairs(input_data)
        generated_pairs = [p for p in pairs]
        self.assertEqual(expected_pairs, generated_pairs)

    def testZip(self):
        content = {"test_key": "test data"}
        zip_b64 = compress_encode(json.dumps(content))
        decompressed = decompress_decode(zip_b64)
        self.assertEqual(content, json.loads(decompressed))


if __name__ == "__main__":
    unittest.main()



import logging
import os
import time
from datetime import datetime
from unittest import TestCase

import pymongo

from mcrit.libs.mongoqueue import MongoQueue
from mcrit.config.QueueConfig import QueueConfig

from .context import config

LOG = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO, format="%(asctime)-15s %(message)s")
logging.disable(logging.CRITICAL)


### Added mongo attribute
import pytest


@pytest.mark.mongo
class MongoQueueTest(TestCase):
    def setUp(self):
        self.client = pymongo.MongoClient(os.environ.get("TEST_MONGODB"))
        queue_config = QueueConfig()
        queue_config.QUEUE_MONGODB_DBNAME = "test_queue"
        queue_config.QUEUE_MONGODB_COLLECTION_NAME = "queue_1"
        self.queue = MongoQueue(queue_config, "consumer_1")

    def tearDown(self):
        self.client.drop_database("test_queue")

    def assert_job_equal(self, job, data):
        for k, v in data.items():
            self.assertEqual(job.payload[k], v)

    def test_put_next(self):
        data = {"method": "test_method", "context_id": "alpha", "data": [1, 2, 3], "more-data": time.time()}
        self.queue.put(dict(data))
        job = self.queue.next()
        self.assert_job_equal(job, data)

    def test_get_empty_queue(self):
        job = self.queue.next()
        self.assertEqual(job, None)

    def test_priority(self):
        self.queue.put({"method": "test_method", "name": "alice"}, priority=1)
        self.queue.put({"method": "test_method", "name": "bob"}, priority=2)
        self.queue.put({"method": "test_method", "name": "mike"}, priority=0)

        self.assertEqual(
            ["bob", "alice", "mike"],
            [self.queue.next().payload["name"], self.queue.next().payload["name"], self.queue.next().payload["name"]],
        )

    def test_complete(self):
        data = {"method": "test_method", "context_id": "alpha", "data": [1, 2, 3], "more-data": datetime.now()}

        self.queue.put(data)
        self.assertEqual(self.queue.size(), 1)
        job = self.queue.next()
        job.complete()
        self.assertEqual(self.queue.size(), 0)

    def test_release(self):
        data = {"method": "test_method", "context_id": "alpha", "data": [1, 2, 3], "more-data": time.time()}

        self.queue.put(data)
        job = self.queue.next()
        job.release()
        self.assertEqual(self.queue.size(), 1)
        job = self.queue.next()
        self.assert_job_equal(job, data)

    def test_error(self):
        pass

    def test_progress(self):
        pass

    def test_stats(self):

        for i in range(5):
            data = {"method": "test_method", "context_id": "alpha", "data": [1, 2, 3], "more-data": time.time()}
            self.queue.put(data)
        job = self.queue.next()
        job.error("problem")
        stats = self.queue.stats()
        self.assertEqual({"available": 5, "total": 5, "locked": 0, "errors": 0}, stats)

    def test_context_manager_error(self):
        self.queue.put({"method": "test_method", "context_id": "alpha", "data": [1, 2, 3], "more-data": time.time()})
        job = self.queue.next()
        try:
            with job as data:
                self.assertEqual(data["payload"]["method"], "test_method")
                # Item is returned to the queue on error
                raise SyntaxError
        except SyntaxError:
            pass

        job = self.queue.next()
        self.assertEqual(job.attempts_left, self.queue.max_attempts-1)

    def test_context_manager_complete(self):
        self.queue.put({"method": "test_method", "context_id": "alpha", "data": [1, 2, 3], "more-data": time.time()})
        job = self.queue.next()
        with job as data:
            self.assertEqual(data["payload"]["method"], "test_method")
        job = self.queue.next()
        self.assertEqual(job, None)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

import os
import re
import uuid
import json
import time
import logging
import hashlib
from datetime import datetime, timedelta
from collections import defaultdict
from itertools import zip_longest
from multiprocessing import Pool, cpu_count
import subprocess
from typing import Dict, List, Optional, TYPE_CHECKING, Tuple

import tqdm
from smda.common.BinaryInfo import BinaryInfo
from smda.common.SmdaFunction import SmdaFunction
from smda.common.SmdaReport import SmdaReport
from smda.Disassembler import Disassembler
from smda.SmdaConfig import SmdaConfig
from smda.common.SmdaInstruction import SmdaInstruction
from smda.intel.IntelInstructionEscaper import IntelInstructionEscaper

from mcrit.Worker import Worker
from mcrit.config.McritConfig import McritConfig
from mcrit.config.MinHashConfig import MinHashConfig
from mcrit.config.ShinglerConfig import ShinglerConfig
from mcrit.config.QueueConfig import QueueConfig
from mcrit.config.McritConfig import McritConfig
from mcrit.config.StorageConfig import StorageConfig
from mcrit.matchers.MatcherCross import MatcherCross
from mcrit.matchers.MatcherQuery import MatcherQuery
from mcrit.matchers.MatcherSample import MatcherSample
from mcrit.matchers.MatcherVs import MatcherVs
from mcrit.minhash.MinHasher import MinHasher
from mcrit.queue.LocalQueue import Job
from mcrit.queue.QueueFactory import QueueFactory
from mcrit.queue.QueueRemoteCalls import NoProgressReporter, QueueRemoteCallee, Remote, JobProgressReporter
from mcrit.storage.SampleEntry import SampleEntry
from mcrit.storage.StorageFactory import StorageFactory

if TYPE_CHECKING:
    from mcrit.storage.StorageInterface import StorageInterface

logging.basicConfig(level=logging.INFO)
LOGGER = logging.getLogger(__name__)


class SpawningWorker(Worker):
    def __init__(self, queue=None, config=None, storage: Optional["StorageInterface"] = None, profiling=False):
        self._worker_id = f"Worker-{uuid.uuid4()}"
        LOGGER.info(f"Starting as spawning worker: {self._worker_id}")
        if config is None:
            config = McritConfig()

        if not queue:
            queue = QueueFactory().getQueue(config, consumer_id=self._worker_id)

        if profiling:
            print("[!] Running as profiled application.")
            profiling_path = os.path.abspath(os.path.join(os.path.dirname(os.path.abspath(__file__ )), "..", "profiler"))
            os.makedirs(profiling_path, exist_ok=True)
        else:
            profiling_path = None
        super().__init__(queue=queue, config=config, storage=storage, profiling=profiling)

        self.config = config
        self._storage_config = config.STORAGE_CONFIG
        self._minhash_config = config.MINHASH_CONFIG
        self._shingler_config = config.SHINGLER_CONFIG
        self._queue_config = config.QUEUE_CONFIG
        self.minhasher = MinHasher(config.MINHASH_CONFIG, config.SHINGLER_CONFIG)
        if storage:
            self._storage = storage
        else:
            self._storage = StorageFactory.getStorage(config)

    def  __enter__(self):
        return self

    def  __exit__(self, *args):
        # TODO unregister our worker_id from all in-progress jobs found in the queue
        self.queue.unregisterWorker()
        self.queue.release_all_jobs()

    #### NO REDIRECTION: SPAWM SINGLE JOB WORKERS INSTEAD ###
        
    def _executeJobPayload(self, job_payload, job):
        # instead of execution within our own context, spawn a new process as worker for this job payload
        console_handle = subprocess.Popen(["python", "-m", "mcrit", "singlejobworker", "--job_id", str(job.job_id)], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        # extract result_id from console_output
        result_id = None
        try:
            stdout_result, stderr_result = console_handle.communicate(timeout=self._queue_config.QUEUE_SPAWNINGWORKER_CHILDREN_TIMEOUT)
            stdout_result = stdout_result.strip().decode("utf-8")
             # TODO: log output from subprocess in the order it arrived
            # instead of the split to stdout, stderr
            if stdout_result:
                LOGGER.info("STDOUT logs from subprocess: %s", stdout_result)
            if stderr_result:
                stderr_result = stderr_result.strip().decode("utf-8")
                LOGGER.info("STDERR logs from subprocess: %s", stderr_result)

            last_line = stdout_result.split("\n")[-1]
            # successful output should be just the result_id in a single line
            match = re.match("(?P<result_id>[0-9a-fA-F]{24})", last_line)
            if match:
                result_id = match.group("result_id")
        except subprocess.TimeoutExpired:
            LOGGER.error(f"Job {str(job.job_id)} running as child from SpawningWorker timed out during processing.")
        return result_id

    def _executeJob(self, job):
        if time.time() - self.t_last_cleanup >= self.queue.clean_interval:
            self.queue.clean()
            self.t_last_cleanup = time.time()
        try:
            result_id = None
            with job as j:
                LOGGER.info("Processing Remote Job: %s", job)
                result_id = self._executeJobPayload(j["payload"], job)
            if result_id:
                # result should have already been persisted by the child process,we repeat it here to close the job for the queue
                job.result = result_id
                LOGGER.info("Finished Remote Job with result_id: %s", result_id)
            else:
                LOGGER.info("Failed Running Remote Job: %s", job)
        except Exception as exc:
            pass

    def run(self):
        self._alive = True
        while self._alive:
            job = self.queue.next()
            if job:
                LOGGER.debug("Found job")
                self._executeJob(job)
            else:
                time.sleep(0.1)



#!/usr/bin/python

import json
import logging
import os
import unittest

from mcrit.shinglers.AbstractShingler import AbstractShingler

from .context import config

LOG = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO, format="%(asctime)-15s %(message)s")
logging.disable(logging.CRITICAL)


class MinHashingTestSuite(unittest.TestCase):
    """Run a full example on a memory dump"""

    def testBucketing(self):
        shingler = AbstractShingler("dummy")
        test_pairs = {
            0: (0, 0, 1),
            2: (1, 2, 3),
            3: (2, 3, 4),
            4: (3, 4, 6),
            5: (4, 6, 8),
            11: (10, 12, 14),
            12: (10, 12, 14),
            16: (14, 16, 20),
            18: (16, 20, 24),
            29: (28, 32, 36),
            32: (28, 32, 36),
            33: (32, 36, 40),
            59: (56, 60, 64),
            89: (88, 96, 104),
            1197: (1184, 1216, 1248),
        }
        for input_value, expected in sorted(test_pairs.items()):
            self.assertEqual(expected, shingler._getLogBucketRange(input_value))


if __name__ == "__main__":
    unittest.main()



import re
import os
import sys


def runWorker(profiling=False):
    from mcrit.Worker import Worker
    with Worker(profiling=profiling) as worker:
        worker.run()


def runSpawningWorker(profiling=False):
    from mcrit.SpawningWorker import SpawningWorker
    with SpawningWorker(profiling=profiling) as spawning_worker:
        spawning_worker.run()


def runSingleJobWorker(job_id, profiling=False):
    from mcrit.SingleJobWorker import SingleJobWorker
    with SingleJobWorker(job_id, profiling=profiling) as single_job_worker:
        single_job_worker.run()


def runServer(profiling=False, uses_gunicorn=False):
    import platform
    from waitress import serve
    from mcrit.server.wsgi import app
    from mcrit.config.GunicornConfig import GunicornConfig
    try:
        import gunicorn
        from gunicorn.app.base import BaseApplication
    except Exception:
        gunicorn = None

    class gunicornServer(BaseApplication):
        def __init__(self, app):
            self.app = app
            super().__init__()
        
        def load_config(self):
            for key, value in GunicornConfig().toDict().items():
                try:
                    self.cfg.set(key.lower(), value)
                except AttributeError:
                    continue

        def load(self):
            return self.app

    wrapped_app = app
    if profiling:
        print("[!] Running as profiled application.")
        from werkzeug.middleware.profiler import ProfilerMiddleware
        profile_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "..", "profiler")
        os.makedirs(profile_dir, exist_ok=True)
        wrapped_app = ProfilerMiddleware(
            wrapped_app,
            restrictions=[30],
            profile_dir=profile_dir,
            filename_format="{method}-{path}-{time:.0f}-{elapsed:.0f}ms.prof",
        )
    
    platform = platform.system().lower()
    if platform == "linux" and gunicorn is not None and (GunicornConfig().USE_GUNICORN or uses_gunicorn):
        print("[!] Detected linux platform and gunicorn availability. Using gunicorn deployment.")
        gunicornServer(wrapped_app).run()
        sys.exit()
    elif platform == "windows":
        print("[!] Detected windows platform. Using waitress deployment.")
    else:
        print("[!] Could not determine platform, gunicorn not available or activated. Defaulting to waitress deployment.")
    # TODO consider allowing an argument to pass an configuration for initial setup of the instance
    serve(wrapped_app, listen="*:8000")


def runClient():
    from mcrit.client.McritConsole import McritConsole
    # we will re-parse arguments with more detail in McritConsole to keep this file concise
    console_client = McritConsole()
    console_client.run()


def main():
    # do not use argparse for processing here to allow using argparse help for the more intricate things in McritConsole
    if len(sys.argv) >= 2:
        is_profiling = False
        uses_gunicorn = False
        if len(sys.argv) >= 3 and "--profile" in sys.argv[2:]:
            is_profiling = True
        if len(sys.argv) >= 3 and "--gunicorn" in sys.argv[2:]:
            uses_gunicorn = True
        if sys.argv[1] == "server":
            runServer(profiling=is_profiling, uses_gunicorn=uses_gunicorn)
        elif sys.argv[1] == "worker":
            runWorker(profiling=is_profiling)
        elif sys.argv[1] == "spawningworker":
            runSpawningWorker(profiling=is_profiling)
        elif sys.argv[1] == "singlejobworker":
            # parse job_id directly from full command line string
            job_id = None
            match = re.search("--job_id (?P<job_id>[0-9a-fA-F]{24})", " ".join(sys.argv))
            if match:
                job_id = match.group("job_id")
                runSingleJobWorker(job_id, profiling=is_profiling)
            else:
                print("Was not able to recognize a job_id (24char hex string), did you pass it using --job_id?")
        elif sys.argv[1] == "client":
            runClient()
        else:
            print("Unrecognized command, please use {{server, worker, spawningworker, client}}")
    else:
        print(f"Usage: {sys.argv[0]} {{server, worker, spawningworker, client}}")
        print("Optionally use --profiling for {server, worker, spawningworker}")
        sys.exit()

if __name__ == "__main__":
    main()



#!/usr/bin/python

import logging
import unittest

from mcrit.index.MinHashIndex import MinHashIndex
from mcrit.libs.utility import generate_unique_pairs

from .context import config

LOG = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO, format="%(asctime)-15s %(message)s")
logging.disable(logging.CRITICAL)


class MinHashIndexTestSuite(unittest.TestCase):
    """Run a full example on a memory dump"""

    def testMinHashIndexInit(self):
        index = MinHashIndex(config)

    def testCandidatePairGenerator(self):
        test_data = [([], []), ([1, 2, 3, 4], [(1, 2), (1, 3), (1, 4), (2, 3), (2, 4), (3, 4)])]
        for data in test_data:
            generated_all_candidates = [pair for pair in generate_unique_pairs(data[0])]
            self.assertEqual(data[1], generated_all_candidates)


if __name__ == "__main__":
    unittest.main()



import json
import logging
import os
import time
from datetime import datetime
from unittest import TestCase, main

import pymongo
from mcrit.config.McritConfig import McritConfig
from mcrit.config.StorageConfig import StorageConfig
from mcrit.config.MinHashConfig import MinHashConfig
from mcrit.config.ShinglerConfig import ShinglerConfig
from mcrit.config.QueueConfig import QueueConfig
from mcrit.minhash.MinHash import MinHash
from mcrit.storage.FunctionEntry import FunctionEntry
from mcrit.storage.SampleEntry import SampleEntry
from mcrit.storage.StorageFactory import StorageFactory
from smda.common.SmdaReport import SmdaReport

from .context import config

LOG = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO, format="%(asctime)-15s %(message)s")
logging.disable(logging.CRITICAL)


class MemoryStorageTest(TestCase):
    def setUp(self):
        self._storage_config = StorageConfig(
            STORAGE_METHOD=StorageFactory.STORAGE_METHOD_MEMORY,
            STORAGE_DROP_DISASSEMBLY=False,
        )
        mcrit_config = McritConfig()
        mcrit_config.STORAGE_CONFIG = self._storage_config
        mcrit_config.MINHASH_CONFIG = MinHashConfig()
        mcrit_config.MINHASH_CONFIG.MINHASH_SIGNATURE_LENGTH = 10
        mcrit_config.MINHASH_CONFIG.MINHASH_SIGNATURE_BITS = 8
        mcrit_config.SHINGLER_CONFIG = ShinglerConfig()
        mcrit_config.QUEUE_CONFIG = QueueConfig()
        self.storage = StorageFactory.getStorage(mcrit_config)
        # get example_file_path
        THIS_FILE_PATH = str(os.path.abspath(__file__))
        PROJECT_ROOT = str(os.path.abspath(os.sep.join([THIS_FILE_PATH, "..", ".."])))
        self.example_file_path = os.sep.join([PROJECT_ROOT, "tests", "example_report.smda"])

    def tearDown(self):
        self.storage.clearStorage()

    def testBasicStorageUsage(self):
        self.storage.clearStorage()
        smda_report = SmdaReport.fromFile(self.example_file_path)
        self.storage.addSmdaReport(smda_report)
        stats = self.storage.getStats()
        self.assertEqual(1, stats["num_samples"])
        self.assertEqual(10, stats["num_functions"])
        self.assertEqual(10, stats["num_pichashes"])

    def testFamilyHandling(self):
        self.storage.clearStorage()
        self.storage.addFamily("family_1")
        self.storage.addFamily("family_2")
        id_3 = self.storage.addFamily("family_3")
        id_3_again = self.storage.addFamily("family_3")
        self.assertEqual(id_3, 3)
        self.assertEqual(id_3_again, 3)

        # family 0 is default: ""
        self.assertEqual(0, self.storage.getFamilyId(""))
        self.assertEqual("", self.storage.getFamily(0).family_name)
        self.assertEqual(4, len(self.storage.getFamilyIds()))
        self.assertEqual("family_1", self.storage.getFamily(1).family_name)
        self.assertEqual(3, self.storage.getFamilyId("family_3"))
        self.assertIsNone(self.storage.getFamily(1000))
        self.assertIsNone(self.storage.getFamilyId("nonexistent"))

        # family modification
        self.storage.modifyFamily(1, {"family_name": "family_1a"})
        self.assertEqual(None, self.storage.getFamilyId("family_1"))
        self.assertEqual(4, self.storage.getFamilyId("family_1a"))
        # family deletion
        self.storage.deleteFamily(4)
        self.assertEqual(None, self.storage.getFamilyId("family_1a"))

    def testSampleHandling(self):
        self.storage.clearStorage()
        # TODO: different samples required, because addSmdaReport wont accept identical hashes
        with open(self.example_file_path, "r") as fjson:
            smda_json = json.load(fjson)
        smda_report_a = SmdaReport.fromDict(smda_json)
        smda_report_a.family = "family_1"
        smda_report_a.is_library = False
        smda_report_a.sha256 = 64 * "a"
        smda_report_b = SmdaReport.fromDict(smda_json)
        smda_report_b.family = "family_1"
        smda_report_b.is_library = False
        smda_report_b.sha256 = 64 * "b"
        smda_report_c = SmdaReport.fromDict(smda_json)
        smda_report_c.family = "family_2"
        smda_report_c.is_library = False
        smda_report_c.sha256 = 64 * "c"
        smda_report_d = SmdaReport.fromDict(smda_json)
        smda_report_d.family = "family_3"
        smda_report_d.is_library = True
        smda_report_d.version = "3.42"
        smda_report_d.sha256 = 64 * "d"
        self.storage.addSmdaReport(smda_report_a)
        self.storage.addSmdaReport(smda_report_b)
        self.storage.addSmdaReport(smda_report_c)
        sample_entry_d = self.storage.addSmdaReport(smda_report_d)
        # produce minhashes for  later testing of clean deletion
        unhashed_function_ids = self.storage.getUnhashedFunctions(None, only_function_ids=True)
        unhashed_functions = self.storage.getUnhashedFunctions(unhashed_function_ids)
        # minhashes = self.calculateMinHashes(unhashed_functions)
        from mcrit.minhash.MinHasher import MinHasher
        from smda.common.BinaryInfo import BinaryInfo
        from smda.common.SmdaFunction import SmdaFunction
        minhasher = MinHasher(MinHashConfig(), ShinglerConfig())
        minhashes = []
        smda_functions = []
        for func in unhashed_functions:
            binary_info = BinaryInfo(b"")
            binary_info.architecture = func.architecture
            smda_functions.append((func.function_id, SmdaFunction.fromDict(func.xcfg, binary_info=binary_info)))
        smda_functions = [
            (function_id, smda_function)
            for function_id, smda_function in smda_functions
            if minhasher.isMinHashableFunction(smda_function)
        ]
        for smda_function in smda_functions:
            minhashes.append(minhasher.calculateMinHashFromStorage(smda_function))
        if minhashes:
            self.storage.addMinHashes(minhashes)
        # start tests
        self.assertIsInstance(sample_entry_d, SampleEntry)
        self.assertEqual(sample_entry_d.sample_id, 3)
        self.assertEqual(None, self.storage.addSmdaReport(smda_report_d))

        self.assertEqual([0, 1, 2, 3], self.storage.getSampleIds())
        self.assertTrue(self.storage.isSampleId(0))
        self.assertFalse(self.storage.isSampleId(4))
        self.assertEqual(None, self.storage.getSampleById(4))
        self.assertEqual(2, self.storage.getSampleById(2).sample_id)
        self.assertEqual(None, self.storage.getSampleIdByFunctionId(40))
        self.assertEqual(3, self.storage.getSampleIdByFunctionId(30))
        self.assertEqual(None, self.storage.getSamplesByFamilyId(4))
        self.assertEqual([0, 1], [s.sample_id for s in self.storage.getSamplesByFamilyId(1)])
        self.assertEqual(None, self.storage.getLibraryInfoForSampleId(2))
        self.assertEqual({"family": "family_3", "version": "3.42"}, self.storage.getLibraryInfoForSampleId(3))

        self.assertEqual(None, self.storage.getLibraryInfoForSampleId(1000))

        self.assertEqual(0, self.storage.getSampleBySha256(64* "a").sample_id)
        self.assertEqual(None, self.storage.getSampleBySha256(64* "z"))

        # test modifications
        self.storage.modifySample(3, {"family_name": "changed_family", "version": "new_version", "component": "new_component", "is_library": True})
        self.assertEqual("changed_family", self.storage.getSampleById(3).family)
        self.assertEqual("new_version", self.storage.getSampleById(3).version)
        self.assertEqual("new_component", self.storage.getSampleById(3).component)
        self.assertEqual(True, self.storage.getSampleById(3).is_library)

        # test deletions
        self.assertFalse(self.storage.deleteSample(1000))
        functions_to_be_deleted = self.storage.getFunctionsBySampleId(3)
        function_ids_to_be_deleted = [f.function_id for f in functions_to_be_deleted]
        minhashes_of_deleted_functions = [f.getMinHash(minhash_bits=MinHashConfig.MINHASH_SIGNATURE_BITS) for f in functions_to_be_deleted if f.minhash]
        delete_result = self.storage.deleteSample(3)
        self.assertTrue(delete_result)
        self.assertEqual(None, self.storage.getSampleById(3))
        # functions, minhashes will be cascadically deleted
        self.assertEqual(None, self.storage.getSampleIdByFunctionId(30))
        # no function id should be contained in minhash bands
        for minhash in minhashes_of_deleted_functions:
            candidates = self.storage.getCandidatesForMinHash(minhash)
            self.assertTrue(len(set(candidates).intersection(set(function_ids_to_be_deleted))) == 0)
        new_report_d = self.storage.addSmdaReport(smda_report_d)
        self.assertIsNotNone(new_report_d)
        self.assertEqual(new_report_d.sample_id, 4)
        self.assertTrue(self.storage.isFunctionId(49))

    def testFunctionHandling(self):
        self.storage.clearStorage()
        # TODO use SmdaReport.fromFile
        with open(self.example_file_path, "r") as fjson:
            smda_json = json.load(fjson)
        smda_report_a = SmdaReport.fromDict(smda_json)
        smda_report_a.sha256 = 64 * "a"
        smda_report_a.family = "family_1"
        smda_report_b = SmdaReport.fromDict(smda_json)
        smda_report_b.family = "family_1"
        smda_report_b.sha256 = 64 * "b"
        self.storage.addSmdaReport(smda_report_a)
        self.storage.addSmdaReport(smda_report_b)

        self.assertTrue(self.storage.isFunctionId(0))
        self.assertTrue(self.storage.isFunctionId(1))
        self.assertFalse(self.storage.isFunctionId(30))
        functions = self.storage.getFunctionsBySampleId(1)
        self.assertIsNotNone(functions)
        self.assertEqual(list(range(10,20)), [entry.function_id for entry in functions])

        function = self.storage.getFunctionById(1, with_xcfg=False)
        self.assertIsNone(function.xcfg)
        function = self.storage.getFunctionById(1, with_xcfg=True)
        self.assertEqual(1, function.function_id)
        self.assertNotEqual({}, function.xcfg)
        self.storage.deleteXcfgForSampleId(function.sample_id)
        function = self.storage.getFunctionById(1, with_xcfg=True)
        self.assertEqual({}, function.xcfg)
        function2 = self.storage.getFunctionById(15, with_xcfg=True)
        self.assertNotEqual({}, function2.xcfg)
        self.storage.deleteXcfgData()
        function2 = self.storage.getFunctionById(15, with_xcfg=True)
        self.assertEqual({}, function2.xcfg)

        self.assertIsNone(self.storage.getFunctionById(1000))
        functions = self.storage.getFunctionsBySampleId(1000)
        self.assertIsNone(functions)

        self.storage.deleteXcfgForSampleId(1000)

    def testHashHandling(self):
        storage_config = StorageConfig()
        storage_config.STORAGE_BANDS = {2: 2, 3: 8}
        storage_config.STORAGE_BAND_SEED = 0

        self.storage.clearStorage()
        with open(self.example_file_path, "r") as fjson:
            smda_json = json.load(fjson)
        smda_report_a = SmdaReport.fromDict(smda_json)
        smda_report_a.sha256 = 64 * "a"
        smda_report_a.family = "family_1"
        smda_report_b = SmdaReport.fromDict(smda_json)
        smda_report_b.family = "family_1"
        smda_report_b.sha256 = 64 * "b"
        self.storage.addSmdaReport(smda_report_a)
        self.storage.addSmdaReport(smda_report_b)

        # pichash tests
        sample_entry = SampleEntry(smda_report_a, sample_id=1, family_id=1)
        function_entry = FunctionEntry(sample_entry, smda_report_a.getFunction(356), 1)
        # Will this work?
        initial_pichash = function_entry.pichash
        pichashes = self.storage.getPicHashMatchesByFunctionId(1)
        self.assertTrue(initial_pichash in pichashes)
        family_sample_and_function_ids = self.storage.getMatchesForPicHash(initial_pichash)
        self.assertTrue(self.storage.isPicHash(initial_pichash))
        self.assertEqual(set([(1, 0, 1), (1, 1, 11)]), family_sample_and_function_ids)

        not_a_pichash = 0
        self.assertEqual(set(), self.storage.getMatchesForPicHash(not_a_pichash))

        pichashes_by_function_ids = self.storage.getPicHashMatchesByFunctionIds(list(range(10,20)))
        pichashes_by_sample_id = self.storage.getPicHashMatchesBySampleId(1)
        self.assertEqual(pichashes_by_function_ids, pichashes_by_sample_id)

        self.assertIsNone(self.storage.getPicHashMatchesBySampleId(1000))
        self.assertIsNone(self.storage.getPicHashMatchesByFunctionId(1000))
        self.assertEqual(pichashes, self.storage.getPicHashMatchesByFunctionIds([1, 1, 1000]))

        # minhash tests
        # TODO check if MinHash initialization works
        minhash_a = MinHash(
            function_id=1, minhash_signature=[0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39], minhash_bits=8
        )
        minhash_b = MinHash(
            function_id=3, minhash_signature=[0x30, 0x31, 0x30, 0x33, 0x30, 0x30, 0x30, 0x37, 0x38, 0x39], minhash_bits=8
        )
        function_entry = self.storage.getFunctionById(1)
        self.assertEqual(b"", function_entry.minhash)
        status = self.storage.addMinHash(minhash_a)
        self.assertTrue(status)
        self.storage.addMinHash(minhash_b)
        function_entry = self.storage.getFunctionById(1)
        minhash_queried = self.storage.getMinHashByFunctionId(1)
        self.assertEqual(minhash_a.getMinHash(), minhash_queried)
        minhash_queried = self.storage.getMinHashByFunctionId(3)
        self.assertEqual(minhash_b.getMinHash(), minhash_queried)

        self.assertFalse(self.storage.addMinHash(MinHash(function_id=1000)))
        self.assertFalse(self.storage.addMinHash(MinHash(function_id=None)))

        self.assertEqual(None, self.storage.getMinHashByFunctionId(1000))

        # minhash band tests
        candidates = self.storage.getCandidatesForMinHash(minhash_a)
        self.assertEqual(set([1, 3]), candidates)

        candidates = self.storage.getCandidatesForMinHashes({1000: minhash_a})
        self.assertEqual({1000: set([1, 3])}, candidates)

        # band rebuild test
        num_reindexed_minhashes = self.storage.rebuildMinhashBandIndex()
        minhash_c = MinHash(
            function_id=1000, minhash_signature=[0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39], minhash_bits=8
        )
        candidates_after = self.storage.getCandidatesForMinHashes({1000: minhash_c})
        self.assertEqual(candidates, candidates_after)

    def testMatchingCache(self):
        cache = self.storage.createMatchingCache([])
        self.assertTrue(hasattr(cache, "getMinHashByFunctionId"))
        self.assertTrue(hasattr(cache, "getSampleIdByFunctionId"))


### Added mongo attribute
import pytest


@pytest.mark.mongo
class MongoDbStorageTest(MemoryStorageTest):
    def setUp(self):
        mongodb_server = os.environ.get("TEST_MONGODB")
        # assume localhost if no explicit test server is set
        if not mongodb_server:
            mongodb_server = "127.0.0.1"
        self._storage_config = StorageConfig(
            STORAGE_METHOD=StorageFactory.STORAGE_METHOD_MONGODB,
            STORAGE_SERVER=mongodb_server,
            STORAGE_MONGODB_DBNAME="test_mongodbstorage_mcrit",
            STORAGE_DROP_DISASSEMBLY=False,
        )
        mcrit_config = McritConfig()
        mcrit_config.STORAGE_CONFIG = self._storage_config
        mcrit_config.MINHASH_CONFIG = MinHashConfig()
        mcrit_config.MINHASH_CONFIG.MINHASH_SIGNATURE_LENGTH = 10
        mcrit_config.MINHASH_CONFIG.MINHASH_SIGNATURE_BITS = 8
        mcrit_config.SHINGLER_CONFIG = ShinglerConfig()
        mcrit_config.QUEUE_CONFIG = QueueConfig()
        self.storage = StorageFactory.getStorage(mcrit_config)
        # get example_file_path
        THIS_FILE_PATH = str(os.path.abspath(__file__))
        PROJECT_ROOT = str(os.path.abspath(os.sep.join([THIS_FILE_PATH, "..", ".."])))
        self.example_file_path = os.sep.join([PROJECT_ROOT, "tests", "example_report.smda"])


if __name__ == "__main__":
    main()



#!/usr/bin/python

import logging
import os
import unittest

from smda.common.SmdaInstruction import SmdaInstruction
from smda.common.SmdaReport import SmdaReport
from smda.intel.IntelInstructionEscaper import IntelInstructionEscaper

from .context import config

LOG = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO, format="%(asctime)-15s %(message)s")
logging.disable(logging.CRITICAL)


class DisassemblyTestSuite(unittest.TestCase):
    """Run a full example on a memory dump"""

    def testLoadReport(self):
        THIS_FILE_PATH = str(os.path.abspath(__file__))
        PROJECT_ROOT = str(os.path.abspath(os.sep.join([THIS_FILE_PATH, "..", ".."])))
        example_file_path = os.sep.join([PROJECT_ROOT, "tests", "example_report.smda"])
        report = SmdaReport.fromFile(example_file_path)
        self.assertEqual(report.architecture, "intel")
        self.assertEqual(report.bitness, 32)
        self.assertEqual(report.statistics.num_functions, 10)


if __name__ == "__main__":
    unittest.main()
